module Web
  def Web::lib_filename( resource, dirname="missing" )
    if dirname == "missing"
      dirname = File.dirname( Web::parse_trace( caller[1] ).shift )
    end
    File.join( dirname, resource )
  end
  
  def Web::lib_file_contents( filename, dirname="missing" )
    File.open( lib_filename( filename, dirname ), 'r' ) { |f|
      f.read
    }
  end

  def Web::send_lib_file( filename, dirname = "missing" )
    Web::connection::send_file( Web::lib_filename( filename, dirname ) )
  end
  
  def Web::escape(string)
    return nil unless string
    string.gsub(/([^ a-zA-Z0-9_.-]+)/n) do
      '%' + $1.unpack('H2' * $1.size).join('%').upcase
    end.tr(' ', '+')
  end
  
  
  def Web::unescape(string)
    return nil unless string
    string.tr('+', ' ').gsub(/((?:%[0-9a-fA-F]{2})+)/n) do
      [$1.delete('%')].pack('H*')
    end
  end
  

  def Web::escape_html(string)
    return nil unless string
    string.gsub(/&/n, '&amp;').gsub(/\"/n, '&quot;').gsub(/>/n, '&gt;').gsub(/</n, '&lt;')
  end

  # include these methods with Web
  def puts(*args)
    Web::connection.puts(args)
  end
  def print(*args)
    Web::connection.print(*args)
  end
  def write(*args)
    Web::connection.write(*args)
  end
  def escape_html(string)
    Web::escape_html(string)
  end
  def unescape_html(string)
    Web::unescape_html(string)
  end
  def escape_element(string, *elements)
    Web::escape_element(string, *elements)
  end
  def unescape_element(string, *elements)
    Web::unescape_element(string, *elements)
  end
  alias escapeHTML escape_html
  alias html_encode escape_html
  alias unescapeHTML unescape_html
  alias escapeElement escape_element
  alias unescapeElement unescape_element
  def rfc1123_date(time)
    Web::rfc1123_date(time)
  end

  def Web::unescape_html(string)
    return nil unless string
    string.gsub(/&(.*?);/n) do
      match = $1.dup
      case match
      when /\Aamp\z/ni           then '&'
      when /\Aquot\z/ni          then '"'
      when /\Agt\z/ni            then '>'
      when /\Alt\z/ni            then '<'
      when /\A#0*(\d+)\z/n       then
        if Integer($1) < 256
          Integer($1).chr
        else
          if Integer($1) < 65536 and ($KCODE[0] == ?u or $KCODE[0] == ?U)
            [Integer($1)].pack("U")
          else
            "&##{$1};"
          end
        end
      when /\A#x([0-9a-f]+)\z/ni then
        if $1.hex < 256
          $1.hex.chr
        else
          if $1.hex < 65536 and ($KCODE[0] == ?u or $KCODE[0] == ?U)
            [$1.hex].pack("U")
          else
            "&#x#{$1};"
          end
        end
      else
        "&#{match};"
      end
    end
  end
  

  def Web::escape_element(string, *elements)
    return nil unless string
    elements = elements[0] if elements[0].kind_of?(Array)
    unless elements.empty?
      string.gsub(/<\/?(?:#{elements.join("|")})(?!\w)(?:.|\n)*?>/ni) do
        Web::escape_html($&)
      end
    else
      string
    end
  end

  
  def Web::unescape_element(string, *elements)
    return nil unless string
    elements = elements[0] if elements[0].kind_of?(Array)
    unless elements.empty?
      string.gsub(/&lt;\/?(?:#{elements.join("|")})(?!\w)(?:.|\n)*?&gt;/ni) do
        Web::unescape_html($&)
      end
    else
      string
    end
  end
  
  class << self
    alias escapeHTML escape_html
    alias html_encode escape_html
    alias unescapeHTML unescape_html
    alias escapeElement escape_element
    alias unescapeElement unescape_element
  end

  RFC822_DAYS = %w[ Sun Mon Tue Wed Thu Fri Sat ]
  RFC822_MONTHS = %w[ Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec ]
  def Web::rfc1123_date(time)
    t = time.clone.gmtime
    return format("%s, %.2d %s %.4d %.2d:%.2d:%.2d GMT",
                  RFC822_DAYS[t.wday], t.day, RFC822_MONTHS[t.month-1], t.year,
                  t.hour, t.min, t.sec)
  end
  
  def Web::parse_trace(line)
    line =~ /^((\w:){0,1}.*?):(\d+)(:?(.*))$/
    [ $1, $3, $5 ]
  end

end
