#!/usr/bin/ruby

require 'web'
require 'web/phprb'

module Web
  SHEBANG_PATTERN = /^#!.*?[\r\n\f]/
  APPLICATION     = "application.rb"

  Web::config::describe( :load_suffix => "hash of :ext => lambda {|scriptname|...} code handlers, used by Web::load" )
  Web::config[:load_suffix] =       \
  { :rb    => lambda do |script|
      Kernel::load( script )
    end,
    :so    => lambda do |script|    \
      Kernel::load(script)
    end, }
  
  class Template
      attr_accessor :name, :src
      def initialize( name, src )
        @name = name
        # ruby doesn't like mac files ("\r")
        @src = src.gsub( /\r\n|\n|\r/, "\n" )
      end
      
      def evaluate
        eval( self.src )
      end
  
      def include symbol
        Object::instance_eval do
          include( symbol )
        end
      end
      
      def Template::evaluate( name, src )
    	self.new( name, src ).evaluate  
      end
  end
  
  Web::config[:load_suffix].default = lambda do |script|
    rhtml = File.open(script) {|f|f.read}.sub(SHEBANG_PATTERN, '')

    require 'erb'
    compiler = ERB::Compiler.new nil
    compiler.put_cmd = 'Web::write'
    src = compiler.compile(rhtml)
    Web::Template.evaluate( script, src )
  end
  
  def Web::load( scriptname )
    Web::connection.templates ||= [ ]
    Web::connection.templates.push scriptname
    
    scriptname = Web::template_file( scriptname )
    
    unless (File.exists? scriptname)
      raise LoadError.new("Template not found: #{scriptname}")
    end

    Web::local.merge! Web::options[:local] || {}

    unless Web::application_loaded
      Web::application_loaded = true
      Web::load_application(scriptname)
    end

    Web::config[:load_suffix].each do |suffix, block|
      if scriptname =~ /\.#{suffix}$/i
        return block.call( scriptname )
      end
    end
    return Web::config[:load_suffix].default.call( scriptname )

  end

  Web::config[:load_path] ||= ['.']
  def Web::template_file( template )
    template_file = template
    Web::config[:load_path].reverse.each{ |dir|
      full_template_path = File.join( dir, template )
      if (File.exists? full_template_path)
        template_file = full_template_path
      end
    }
    template_file
  end

  def Web::load_application(scriptname)
    search_path = File.expand_path(scriptname)
    while( search_path &&                    \
           !(search_path =~ /^([a-z]:)?\/$/i ) &&             \
           search_path != '.' )
      app = File.join( File::dirname(search_path), APPLICATION )
      
      if File.exists? app
        Web::load app
        search_path = nil
      else
        search_path = File::dirname(search_path)
      end
    end
  end
  
  def Web::ruby_web()
    # remnants of narflets.. do we still want to use this?
    #Web::docroot = ENV["DOCUMENT_ROOT"]
    #$: << "#{Web::docroot}/../src"
    #$: << "#{Web::docroot}/../config"
    
    options = {}
    switches = Web::argv_switches(ARGV)
    switches.each do |arg|
      case arg
      when /^-I/
        $:.unshift arg.sub( /^-I/, '' )
      when /^--cli$/
        options[:noheader] = true
      when /^--ruby$/
        options[:style] = :ruby
      when /^--xml$/
        options[:xml] = true
      else
        #Web::print_usage
        #exit
      end
    end
    
    script = if (File.exists? ENV['PATH_TRANSLATED'].to_s)
               ENV['PATH_TRANSLATED']
             elsif(File.exists?ENV['SCRIPT_FILENAME'].to_s)
               ENV['SCRIPT_FILENAME']
             else
               ARGV[0]
             end
    
    ARGV.shift if (script == ARGV.first )

    Web::open(options) { Web::load( script ) }
  end

  def Web::argv_switches( args )
    args.shift
    switches = []
    found_doubledash = false
    args.delete_if do |e|
      unless found_doubledash
        if e =~ /(^--$)/
          found_doubledash = true
        elsif e =~ /^[^-]/
          found_doubledash = true
          false
        else
          switches.push e
        end
      end
    end
    switches
  end
  
end
