# + modified by Patrick May to support php-style
#   question marks ( <? ... ?> ) instead of
#   the current asp-style ( <% ... %> )
# Tiny eRuby --- ERB2
# Copyright (c) 1999-2000,2002,2003 Masatoshi SEKI 
# You can redistribute it and/or modify it under the same terms as Ruby.

class PHPRB #:nodoc: all
  Revision = '$Date: 2003/12/10 09:07:52 $' 	#'

  def self.version
    "erb.rb [2.0.4 #{PHPRB::Revision.split[1]}]"
  end
end

# PHPRB::Compiler
class PHPRB
  def PHPRB::load( script )
    scriptname = script
    script = if (File.exists? script)
               File.open(scriptname,'r'){|f|f.read} || ""
             else
               scriptname = nil
               script
             end
    
    script.gsub!( Web::SHEBANG_PATTERN, '' )
    
    # I'm keeping the compiler separate from the template
    # class, because I plan to replace the compiler
    #   ~ pat
    script = PHPRB.new(script, nil, Web::options[:xml] ).src
    
    Web::Template.evaluate( scriptname, script )
  end
    
  
  class Compiler
    class PercentLine
      def initialize(str)
        @value = str
      end
      attr_reader :value
      alias :to_s :value
    end
    
    class Scanner

      def initialize(src, xml_mode=nil)
        @src = src
        @stag = nil
        @xml_mode = xml_mode
      end
      attr_accessor :stag
      

      def stag_exp
        open_pattern = if @xml_mode
                         '\?ruby'
                       else
                         '\?(ruby)?'
                       end
        '(<\?\?)|(<' + open_pattern + '=)|(<!---)|(<' + open_pattern + ')'
      end

      def etag_exp
        '(\?\?>)|(--->)|(\?>)'
      end

      begin
        require 'strscan'
        def scan
          stag_reg = /(.*?)(#{self.stag_exp}|\n|\z)/
          etag_reg = /(.*?)(#{self.etag_exp}|\n|\z)/
          scanner = StringScanner.new(@src)
          while ! scanner.eos?
            scanner.scan(@stag ? etag_reg : stag_reg)
            text = scanner[1]
            elem = scanner[2]
            yield(text) unless text.empty?
            yield(elem) unless elem.empty?
          end
        end
      rescue LoadError
        SplitRegexp = /#{self.stag_exp}|#{self.etag_exp}|(\n)/
        def scan
          @src.each do |line|
            line.split(SplitRegexp).each do |token|
              next if token.empty?
              yield(token)
            end
          end
        end
      end

    end

    class Buffer
      def initialize(compiler)
        @compiler = compiler
        @line = []
        @script = ""
        @compiler.pre_cmd.each do |x|
          push(x)
        end
      end
      attr_reader :script

      def push(cmd)
        @line << cmd
      end
      
      def cr
        @script << (@line.join('; '))
        @line = []
        @script << "\n"
      end
      
      def close
        return unless @line
        @compiler.post_cmd.each do |x|
          push(x)
        end
        @script << (@line.join('; '))
        @line = nil
      end
    end

    def compile(s)
      out = Buffer.new(self)

      content = ''
      scanner = make_scanner(s)
      scanner.scan do |token|
        if scanner.stag.nil?
          case token
          when PercentLine
            out.push("#{@put_cmd} #{content.dump}") if content.size > 0
            content = ''
            out.push(token.to_s)
            out.cr
          when :cr
            out.cr
          when '<?', '<?=', '<!---', '<?ruby', '<?ruby='
            scanner.stag = token
            out.push("#{@put_cmd} #{content.dump}") if content.size > 0
            content = ''
          when "\n"
            content << "\n"
            out.push("#{@put_cmd} #{content.dump}")
            out.cr
            content = ''
          when '<??'
            content << '<?'
          else
            content << token
          end
        else
          # comments should include everything inside themselves
          if scanner.stag == '<!---' && token != '--->'
            content << token
          else
            case token
            when '--->'
              scanner.stag = nil
              content = ''
              # out.push("# #{content.dump}")
            when '?>'
              case scanner.stag
              when '<?', '<?ruby'
                if content[-1] == ?\n
                  content.chop!
                  out.push(content)
                  out.cr
                else
                  out.push(content)
                end
              when '<?=', '<?ruby='
                out.push("#{@put_cmd}((#{content}).to_s)")
                #when '<?--'
                # out.push("# #{content.dump}")
              end
              scanner.stag = nil
              content = ''
            when '??>'
              content << '?>'
            else
              content << token
            end
          end
        end
      end
      out.push("#{@put_cmd} #{content.dump}") if content.size > 0
      out.close
      out.script
    end

    def prepare_trim_mode(mode)
      case mode
      when 1
        return [false, '>']
      when 2
        return [false, '<>']
      when 0
        return [false, nil]
      when String
        perc = mode.include?('?')
        if mode.include?('-')
          return [perc, '-']
        elsif mode.include?('<>')
          return [perc, '<>']
        elsif mode.include?('>')
          return [perc, '>']
        else
          [perc, nil]
        end
      else
        return [false, nil]
      end
    end
    
    def make_scanner(src)
      Scanner.new(src, @xml_mode)
    end
    
    def initialize(xml_mode=nil)
      @xml_mode = xml_mode
      @put_cmd = 'Web::print'
      @pre_cmd = []
      @post_cmd = []
    end
    attr_accessor :put_cmd, :pre_cmd, :post_cmd
  end
end

# PHPRB
class PHPRB
  def initialize(str, safe_level=nil, xml_mode=nil )
    @safe_level = safe_level
    compiler = PHPRB::Compiler.new(xml_mode)
    
    #, eoutvar='_erbout')
    #set_eoutvar(compiler, eoutvar)
    #def set_eoutvar(compiler, eoutvar = '_erbout')
    #compiler.put_cmd = "#{eoutvar}.concat"
    #
    #cmd = []
    #cmd.push "#{eoutvar} = ''"
    #
    #compiler.pre_cmd = cmd
    #
    #cmd = []
    #cmd.push(eoutvar)
    #
    #compiler.post_cmd = cmd
    #end

    compiler.put_cmd = "Web::write"
    #cmd = []
    #cmd.push "#{eoutvar} = ''"
    #
    #compiler.pre_cmd = cmd
    #
    #cmd = []
    #cmd.push(eoutvar)
    #
    #compiler.post_cmd = cmd

    
    @src = compiler.compile(str)
    @filename = nil
  end
  attr_reader :src
  attr_accessor :filename
  
  def run(b=TOPLEVEL_BINDING)
    print self.result(b)
  end
  
  def result(b=TOPLEVEL_BINDING)
    if @safe_level
      th = Thread.start { 
        $SAFE = @safe_level
        eval(@src, b)
      }
      return th.value
    else
      return eval(@src, b, (@filename || '(erb)'), 1)
    end
  end
  
  def def_method(mod, methodname, fname='(PHPRB)')
    mod.module_eval("def #{methodname}\n" + self.src + "\nend\n", fname, 0)
  end
  
  def def_module(methodname='erb')
    mod = Module.new
    def_method(mod, methodname)
    mod
  end
  
  def def_class(superklass=Object, methodname='result')
    cls = Class.new(superklass)
    def_method(cls, methodname)
    cls
  end
end

# PHPRB::Util
class PHPRB
  module Util
    public
    def html_escape(s)
      s.to_s.gsub(/&/, "&amp;").gsub(/\"/, "&quot;").gsub(/>/, "&gt;").gsub(/</, "&lt;")
    end
    alias h html_escape
    
    def url_encode(s)
      s.to_s.gsub(/[^a-zA-Z0-9_\-.]/n){ sprintf("%%%02X", $&.unpack("C")[0]) }
    end
    alias u url_encode
  end
end

# PHPRB::DefMethod
class PHPRB
  module DefMethod
    public
    def def_erb_method(methodname, erb)
      if erb.kind_of? String
        fname = erb
        File.open(fname) {|f| erb = PHPRB.new(f.read) }
        erb.def_method(self, methodname, fname)
      else
        erb.def_method(self, methodname)
      end
    end
    module_function :def_erb_method
  end
end
