module Web
  module Install
    class Apache
      require 'fileutils'
      include FileUtils
      
      def Apache::install( install_dir, httpd_conf )
        self.new(install_dir, httpd_conf).install
      end
    
      attr_accessor :install_dir, :httpd_conf
      def initialize( install_dir, httpd_conf )
        @install_dir = install_dir
        @httpd_conf  = httpd_conf
      end
      
      def apache_root
        File.dirname( File.dirname( httpd_conf ) )
      end
      
      def apache
        [apache_root + "/Apache.exe",
         apache_root + "/bin/Apache.exe" ].find do |f|
          File.exists? f
        end
      end
      
      def test_config_cmd
        apache + " -t" if apache
      end
      
      def version
        case apache
        when apache_root + "/Apache.exe"
          1
        when apache_root + "/bin/Apache.exe"
          2
        end
      end
      
      def backup
      	"#{httpd_conf}.ruby-web.backup.#{Time::now::strftime('%Y.%m.%d') }"
      end
      
      def temp
        "#{httpd_conf}.ruby-web.temp"
      end
      
      def ruby_conf
      	unless @ruby_conf
          @ruby_conf = Array.new
          @ruby_conf.push "AddHandler cgi-script .rb .rhtml"
        end
        @ruby_conf
      end
    
      def install
        puts "******************************************",
             "* Installing .rhtml and .rb mappings     *",
             "* for Apache                             *",
             "******************************************"    
	
	new_conf = ''
	lines = File.open( httpd_conf, "r" ) {|f| f.read }
	lines.each do |line|
	  if line =~ /index.html/ && ! line =~ /index.rhtml/
	    line.gsub!(/index.html/,'index.html index.php')
	  end
	  if line =~ /AddHandler cgi-script.*(\.rb|\.rhtml)/
	    ruby_conf.delete_at(0)
	  end
	  new_conf += line + "\n"
	end
	
	cp( httpd_conf, backup )
           
        config_file_changed = false
      
        if ruby_conf.empty?
          # apparently there is nothing to do...
        else
	  new_conf += "\n\n################# Ruby Web ###################\n"
	  new_conf += ruby_conf.join "\n"
	
	  File.open(temp, "w") do |f|
            f.write new_conf
          end
        
          cp( temp, httpd_conf )
        
          config_file_changed = true

          # ----------- Restart Apache ------------
          unless system(test_config_cmd)
            puts("[ruby-web #{$0}]: Unable that the Apache webserver is running correcly with the new module and the configuration file changes using the '#{test_config_cmd}' command.  Ruby-Web itself has been installed sucessfully, but it has not been activated.  Your old #{httpd_conf} file is untouched...")
            puts "[ruby-web #{$0}]: The modified config file is available in '#{temp}' so you can examine it and try to activate Ruby-Web"
          
            cp( backup, conf )
          else
            # restart apache
            system('net stop apache')
	    system('net start apache')
	  end
	end
      end
    end
  end
end

if __FILE__ == $0
  begin
    Web::Install::Apache::install( *ARGV )
  rescue Exception
    puts $!
    puts $!.backtrace
    sleep
  end
end

__END__

<?php

$lines = file($HTTPD_CONF);
foreach($lines as $k => $v) {
	if (strpos($v,'index.html') !== false) {
		if (strpos($v,'index.php') == false) {
			$lines[$k] = str_replace('index.html','index.html index.php',$v);
		}
	}
	
	if (preg_match('/LoadModule\s+php4_module/i',$v)) {
		$lines[$k] = $insarr[0];
		$insarr[0] = '';
	}
	
	if (preg_match('/AddModule\s+mod_php4.c/i',$v)) {
		$lines[$k] = $insarr[1];
		$insarr[1] = '';
	}
	
	if (preg_match('/AddType application\/x-httpd-php \.php/i',$v)) {
		$lines[$k] = $insarr[2];
		$insarr[2] = '';
	}
	
	if (preg_match('/^\s*DocumentRoot\s+(.*)/',$v,$rootarr)) {
		$root = $rootarr[1];
		if (substr($root,0,1) == '"') $root = trim(substr($root,1));
		if (substr($root,strlen($root)-1) == '"') $root = substr($root,0,strlen($root)-1);
		if (substr($root,strlen($root)-1) == '/') $root = substr($root,0,strlen($root)-1);
		if (substr($root,strlen($root)-1) == '\\') $root = substr($root,0,strlen($root)-1);
	}
}

if ($insarr[0] != '') $lines[] = "\r\n\r\n################# PHP ###################\r\n";
foreach($insarr as $v) $lines[] = $v;

$f = fopen($HTTPD_CONF,'w');
if ($f) {
	foreach($lines as $v) fwrite($f,$v);
	fclose($f);
}

system('net stop apache');
system('net start apache');
sleep(2);

------------------------------
error_reporting(E_ALL);
if (sizeof($argv) <= 2) return;

if (!$HTTPD_CONF) return;

$insarr[] =   "LoadModule php4_module $MAINDIR\\php\\php4apache.dll\r\n";
$insarr[] = "AddModule mod_php4.c\r\n";
$insarr[] = "AddType application/x-httpd-php .php\r\n";


?>