require 'pp'

module Web
  module Install
    class IIS
      
      def IIS::install( install_dir )
        self.new(install_dir).install
      end
    
      attr_accessor :install_dir
      def initialize( install_dir )
        @install_dir = install_dir
      end
      
      def backup
      	"#{httpd_conf}.ruby-web.backup.#{Time::now::strftime('%Y.%m.%d') }"
      end
      
      def install
        puts "******************************************",
             "* Installing .rhtml and .rb mappings     *",
             "* for Microsoft IIS                      *",
             "******************************************"    

	done = Hash.new
	require 'win32ole'
	# http://www.west-wind.com/presentations/WebServerConfig/WebServerConfig.htm
	iis = WIN32OLE.connect 'IIS://LOCALHOST/W3SVC'
	
	iis.each do |site|
	  unless site.name =~ /info|filters/i
	    site.each do |d|
	      if d.name =~ /^root$/i
	        scriptmaps = d.scriptmaps
	  
                scriptmaps.each_with_index do |mapping, i|
	          m = mapping.split(/,/)
	          if m.first =~ /^\.(rb|rhtml)$/
	            done[m.first] = true
	            m[1] = install_dir + "\\bin\\ruby-web.exe"
	            m = m.join(',')
	            scriptmaps[i] = m
	          end
	        end
	
	        ['.rb', '.rhtml'].each do |ext|
	          unless done[ext]
	            scriptmaps[scriptmaps.length] = "#{ext},#{install_dir}\\bin\\ruby-web.exe,1"
	          end
	        end
	    
	        # http://www.microsoft.com/technet/scriptcenter/guide/sas_ads_kbwl.mspx
	        d.PutEx( ADS_PROPERTY_UPDATE, "ScriptMaps", scriptmaps )
	        
	        index_files = d.DefaultDoc
	        index_files += ",index.rhtml" unless index_files =~ /index\.rhtml/
	        index_files += ",index.rb" unless index_files =~ /index\.rb/
	        d.DefaultDoc = index_files
	        
	        d.SetInfo()
	      end
	    end
	  end
	end
	
	#require 'pp'
	#pp iis.scriptmaps
        
        puts "Restarting IIS..."
        unless system('iisreset')
          raise "Error restarting IIS"
        end
      end
      
      ADS_PROPERTY_CLEAR = 1
      ADS_PROPERTY_UPDATE = 2
      ADS_PROPERTY_APPEND = 3
      ADS_PROPERTY_DELETE = 4
      
    end
  end
end

if __FILE__ == $0
  begin
    Web::Install::IIS::install( *ARGV )
  rescue Exception
    puts $!
    puts $!.backtrace
    sleep
  end
end