;!define VERSION "1.0"
!define PROGRAM "ruby-web"
;!define OUTFILE "${PROGRAM}_${VERSION}_win.exe"

!define REG_KEY "Software\Microsoft\Windows\CurrentVersion\Uninstall\ruby-web" ; _${VERSION}
!define UNINSTALLER "uninstall_${PROGRAM}.exe"   ;_${VERSION}

;--------------------------------
;Includes

  !include "MUI.nsh"
  !include "Sections.nsh"

;--------------------------------
;General

  Name "${PROGRAM} v${VERSION}"
  OutFile "${OUTFILE}"

  SetCompressor lzma

  ; Installation directory
  InstallDir c:\ruby
  InstProgressFlags smooth
   
  ; detect Winamp path from uninstall string if available
  InstallDirRegKey HKLM \
            "Software\Microsoft\Windows\CurrentVersion\Uninstall\Ruby" \
          "UninstallString"
          
  ; The text to prompt the user to enter a directory
  DirText "Please select your Ruby Installer path below (you will be able to proceed \
         when Ruby is detected):"
  
  ; here we check to see if this a valid location ie is there a ruby.exe
  ; in the directory?
  Function .onVerifyInstDir
    ;Check for Ruby installation
    IfFileExists $INSTDIR\bin\ruby.exe Good
      Abort
    Good:
  FunctionEnd

;--------------------------------
;Interface Settings

;--------------------------------
;Pages

  ;!insertmacro MUI_PAGE_LICENSE "${NSISDIR}\Docs\Modern UI\License.txt"
  !insertmacro MUI_PAGE_DIRECTORY
  
  !define MUI_PAGE_HEADER_TEXT "Web Server Mappings"
  !define MUI_PAGE_HEADER_SUBTEXT ""
  !define MUI_COMPONENTSPAGE_TEXT_TOP "ruby-web can install ruby support, including mappings for .rb and .rhtml files, into your web server.  Please select the appropriate servers below."
  !define MUI_COMPONENTSPAGE_TEXT_COMPLIST "Select Web Servers:"
  !define MUI_COMPONENTSPAGE_TEXT_DESCRIPTION_TITLE ""
  !insertmacro MUI_PAGE_COMPONENTS

  Page custom SelectApache1Conf
  Page custom SelectApache2Conf
  
  !insertmacro MUI_PAGE_INSTFILES
  
  ;-----------------
  
  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES
  
  ReserveFile "apache1.ini"
  ReserveFile "apache2.ini"
  !insertmacro MUI_RESERVEFILE_INSTALLOPTIONS

  Var apache1conf
  Var apache2conf
  
  Function .onInit
    !insertmacro MUI_INSTALLOPTIONS_EXTRACT "apache1.ini"
    !insertmacro MUI_INSTALLOPTIONS_EXTRACT "apache2.ini"
  FunctionEnd
  
;--------------------------------
;Languages
 
  !insertmacro MUI_LANGUAGE "English"

;--------------------------------
;Installer Sections

; The stuff to install
Section -ruby-web
  SetOverwrite on
  SetOutPath $INSTDIR
  File /r /x *.svn "bin"
  File /r /x *.svn "lib"
  SetOverwrite off

  ;Create uninstaller
  WriteUninstaller "$INSTDIR\${UNINSTALLER}"

  WriteRegStr HKLM "${REG_KEY}" "UninstallString" '"$INSTDIR\${UNINSTALLER}"'
  WriteRegStr HKLM "${REG_KEY}" "DisplayName" "${PROGRAM} ${VERSION} (uninstall)"
  WriteRegStr HKLM "${REG_KEY}" "DisplayVersion" "${VERSION}"
  WriteRegStr HKLM "${REG_KEY}" "URLInfoAbout" "http://www.ruby-web.org"
  WriteRegStr HKLM "${REG_KEY}" "InstallLocation" "$INSTDIR"
  

SectionEnd

  Section /o "IIS" iis
    ExecWait '"$INSTDIR/bin/ruby.exe" "$INSTDIR/lib/ruby/site_ruby/1.8/web/sapi/install/iis.rb" "$INSTDIR"'
  SectionEnd
  
  Section /o "Apache 1.x" apache1
    ExecWait '"$INSTDIR/bin/ruby.exe" "$INSTDIR/lib/ruby/site_ruby/1.8/web/sapi/install/apache.rb" "$INSTDIR" "$apache1conf"'
  SectionEnd

  Section /o "Apache 2.x" apache2
    ExecWait '"$INSTDIR/bin/ruby.exe" "$INSTDIR/lib/ruby/site_ruby/1.8/web/sapi/install/apache.rb" "$INSTDIR" "$apache2conf"'
  SectionEnd

;--------------------------------
;Descriptions

  ;Language strings
  ;LangString DESC_ruby-web ${LANG_ENGLISH} "Upgrade your ruby installation with the ruby-web libraries."
  LangString DESC_iis ${LANG_ENGLISH} "IIS Mappings for .rb and .rhtml files."
  LangString DESC_apache1 ${LANG_ENGLISH} "Apache 1.x Mappings for .rb and .rhtml files."
  LangString DESC_apache2 ${LANG_ENGLISH} "Apache 2.x Mappings for .rb and .rhtml files."

  ;Assign language strings to sections
  !insertmacro MUI_FUNCTION_DESCRIPTION_BEGIN
    ;!insertmacro MUI_DESCRIPTION_TEXT ${-ruby-web} $(DESC_ruby-web)
    !insertmacro MUI_DESCRIPTION_TEXT ${iis} $(DESC_iis)
    !insertmacro MUI_DESCRIPTION_TEXT ${apache1} $(DESC_apache1)
    !insertmacro MUI_DESCRIPTION_TEXT ${apache2} $(DESC_apache2)
  !insertmacro MUI_FUNCTION_DESCRIPTION_END

;--------------------------------
;Uninstaller Section

Section "Uninstall"

  Delete "$INSTDIR\bin\ruby-web.exe"
  Delete "$INSTDIR\lib\ruby\site_ruby\1.8\web.rb"
  RMDir /r "$INSTDIR\lib\ruby\site_ruby\1.8\web"

  DeleteRegKey /ifempty HKLM ${REG_KEY}

  Delete "$INSTDIR\${UNINSTALLER}"  
  
SectionEnd




  Function SelectApache1Conf
    push $R0
    
    SectionGetFlags ${apache1} $R0 
    IntOp $R0 $R0 & ${SF_SELECTED} 
    IntCmp $R0 ${SF_SELECTED} show 
    
    pop $R0
    Abort 
     
    show:
      pop $R0
      !insertmacro MUI_HEADER_TEXT "Web Server Mappings" "Select Apache 1.x httpd.conf"
      !insertmacro MUI_INSTALLOPTIONS_DISPLAY "apache1.ini"
      ReadINIStr $apache1conf "apache1.ini" "Field 2" "State"
  FunctionEnd

  Function SelectApache2Conf
    push $R0
    
    SectionGetFlags ${apache2} $R0 
    IntOp $R0 $R0 & ${SF_SELECTED} 
    IntCmp $R0 ${SF_SELECTED} show 
    
    pop $R0
    Abort 
     
    show:
      pop $R0
      !insertmacro MUI_HEADER_TEXT "Web Server Mappings" "Select Apache 2.x httpd.conf"
      !insertmacro MUI_INSTALLOPTIONS_DISPLAY "apache2.ini"
      ReadINIStr $apache2conf "apache2.ini" "Field 2" "State"
  FunctionEnd


