module Web
    class Action
	def Action.clear_actions
          Thread::current[:actions] = [ ]
	end
    end
end

class TestAction < Test::Unit::TestCase
    def set_up
    end
    
    def teardown
	@output = @cgi = @action = @stored_pattern = nil
	Web::Action.clear_actions
    end

    def output
	@output ||= ""
    end

    def cgi params={}
	Web::CGI.new( :out => output, :params => params )
    end

    def stored_pattern pattern={}
	@stored_pattern ||= pattern
    end

    def action( pattern=stored_pattern, &function )
	@action ||= Web::Action.new( pattern, &function )
    end

    def test_failed_match_on_blank_cgi
	action( "name" => // )
	assert_equal( false, action.applies?(cgi) )
    end

    def test_fails_with_different_params
	action "name" => //
	assert_equal( false, action.applies?(cgi( {"not" => "me"} )) )
    end

    def test_matchs_by_param_present
	action "name" => //
	assert action.applies?(cgi( {"name" => "something"}) )
    end
    
    def test_matches_exact_param
	action "name" => "patrick"
	assert action.applies?(cgi( {"name" => "patrick" } ))
	assert_equal( false, action.applies?(cgi( {"name" => "patrick.may"} ) ))
    end

    def test_matches_pattern
	action "name" => /patrick|tom/
	assert action.applies?(cgi( {"name" => "patrick the great" } ))
	assert action.applies?(cgi( {"name" => "tom the silly" } ))
    end
	
    def test_match_is_not_anal
	action "name" => //
	assert action.applies?(cgi( {"name" => "something", "ignore" => "me"} ))
    end

    def test_runs_block
	action_ran_flag = false
	action( "name" => // ) {
	    action_ran_flag = true
	}

	action.run

	assert action_ran_flag
    end

    def test_guards_block
	Web::Action.new.run
    end

    def test_passes_cgi_in_with_run
	action_ran_flag = false
	action( "name" => // ){ |cgi|
	    action_ran_flag = cgi.key? "flag"
	}

	action.run cgi( "flag" => "it ran" )

	assert action_ran_flag
    end

    def test_pick_action_picks_the_most_specific_action
      one = Web::Action.new( "name" => // ) { assert false }
      second = Web::Action.new( "name" => "patrick" ) { assert true }
      third = Web::Action.new( "name" => /patrick|tom/ ) { assert false }
      
      # for some reason, object ids change with thread local variables
      # assert_equal( second, Web::Action.pick( cgi({ "name" => "patrick"}) ) )
      
      Web::Action.pick( cgi({ "name" => "patrick"}) ).run
    end

    def test_set_default_action
	default = Web::Action.new
	another = Web::Action.new( "this" => "that", "here" => "there" )
	assert_equal( default, Web::Action.pick( cgi ) )
    end

# this test causes no initial default action
#    def test_raises_exception_if_no_action_is_found
#	assert_raises(Web::Exception){ 
#	    Web::Action.pick cgi
#	}
#    end

    def test_doesnt_raise_exception_if_no_exception_is_found
	Web::Action.pick(cgi).run
    end

    def test_blank_action_doesnt_mask_default
	Web::Action.pick(cgi).run

	default = Web::Action.new

	assert_equal( default, Web::Action.pick(cgi) )
    end

    def test_run
      cgi_params = { "name" => "patrick" }
      test_params = {}
      
      Web::Action.new { |cgi|
        test_params = cgi.multiple_params
      }
      
      Web::set_cgi nil
      Web::Action.run :params => cgi_params, :out => ""

      assert_equal( cgi_params, test_params )
    end
    
    def todo_test_subclassing_action

    end
    
end
