class AForm
end

class TestLink < Test::Unit::TestCase
  def test_empty_query
    string = Web::SimpleDispatcher::Link.new( AForm, { "name1" => "value1", "name2" => "value2"  } ).encode
    assert_equal "String", string.class.name
    params = Web::CGI.new(:params => {"link" => string, "link-type" => "Web::SimpleDispatcher::Link"}).typed_params
    assert_equal AForm, params["link"].page
    assert_equal({ "name1" => "value1", "name2" => "value2" },params["link"].query)
  end
end

class MyForm1 < Web::SimpleDispatcher::Template
    def initialize query
    end
end

class MyForm2 < Web::SimpleDispatcher::Template
    def initialize query
	Web.puts "form2"
    end
end

class MyForm3 < Web::SimpleDispatcher::Template
    attr_accessor :var

    def initialize query
	@var = "hello world"
    end
end

class MyForm4 < Web::SimpleDispatcher::Template
    attr_accessor :var

    def initialize query
	@var = "hello world"
    end

    def on_submit form_values
	@var = "goodbye world"
    end
end

class MyForm5 < Web::SimpleDispatcher::Template
    attr_accessor :var

    def initialize query
	@var = "hello world"
    end

    def on_submit form_values
	Web.puts @var
    end
end

class MyForm6 < Web::SimpleDispatcher::Template
    attr_accessor :var

    def initialize query
	@var = "hello world"
    end

    def on_submit form_values
	Web.puts @var

	redirect_to_template MyForm7, "var" => "xxxx"
    end
end

class MyForm7 < Web::SimpleDispatcher::Template
    def initialize query
	Web << query["var"]
    end
end

class TestLinkTemplate < Web::SimpleDispatcher::Template
    attr_accessor :alink
    
    def initialize query
        @alink = Web::SimpleDispatcher::Link.new( MyForm1 , { "a" => "1", "b" => "2" } )
    end
end


class TestForm < Test::Unit::TestCase
    include Web::Testing
    require 'ftools'
    def setup
	if File.exists?( temp_dir + "/test_dir/" )
	  File.rm_rf( temp_dir + "/test_dir")
	end
	Dir.mkdir(temp_dir + "/test_dir/")
	Dir.mkdir(temp_dir + "/test_dir/templates")
	Web.set_docroot(temp_dir + "/test_dir/")
    end
    
    def teardown
	File.rm_rf( temp_dir + "/test_dir")
    end

  def test_displays_template
  # create a file system to put the files in
	file = "Web::SimpleDispatcher.handle_request( :default => MyForm1 )"
	File.open( temp_dir + "/test_dir/app.rb" , "w" ) { |f| f.write(file) }
	file = "<html><body>{$var}</body></html>"
	File.open( temp_dir + "/test_dir/templates/my_form3.html" , "w" ) { |f| f.write(file) }
	do_request( "/app.rb/my_form3" )
	assert_content "<html><body>hello world</body></html>"
    end

    def test_displays_default_template
	# create a file system to put the files in
	file = "Web::SimpleDispatcher.handle_request( :default => MyForm1 )"
	File.open( temp_dir + "/test_dir/app.rb" , "w" ) { |f| f.write(file) }
	file = "<html><body>default</body></html>"
	File.open( temp_dir + "/test_dir/templates/my_form1.html" , "w" ) { |f| f.write(file) }
	do_request( "/app.rb" )
	assert_content "<html><body>default</body></html>"

      # and with a trailing slash
	do_request( "/app.rb/" )
	assert_content "<html><body>default</body></html>"	
	
	# and check that we are there because of a redirect
	assert_equal "/my_form1", Web.path_info
    end

    def test_displays_default_template_with_trailing_slash
	# create a file system to put the files in
	file = "Web::SimpleDispatcher.handle_request( :default => MyForm1 )"
	File.open( temp_dir + "/test_dir/app.rb" , "w" ) { |f| f.write(file) }
	file = "<html><body>default</body></html>"
	File.open( temp_dir + "/test_dir/templates/my_form1.html" , "w" ) { |f| f.write(file) }
    end

    def test_handle_submit
	file = "Web::SimpleDispatcher.handle_request( :default => MyForm1 )"
	File.open( temp_dir + "/test_dir/app.rb" , "w" ) { |f| f.write(file) }
	file = "<html><body>{$var}<narf:form name=\"f\"><input type=submit></narf:form></body></html>"
	File.open( temp_dir + "/test_dir/templates/my_form4.html" , "w" ) { |f| f.write(file) }
	do_request( "/app.rb/my_form4" )
	do_submit( "f" , {})
	assert_content "<html><body>goodbye world<form name=\"f\" method=\"\"><input type=\"hidden\" name=\"__submitted\" value=\"f\"><input type=submit></form></body></html>"
    end

    def test_submit_template_gets_updated
      file = "Web::SimpleDispatcher.handle_request( :default => MyForm1 )"
      File.open( temp_dir + "/test_dir/app.rb" , "w" ) { |f| f.write(file) }
      file = "<html><body>{$var}<narf:form name=f>"+
             "<narf:input type=text name=var></narf:form></body></html>"
      File.open( temp_dir + "/test_dir/templates/my_form5.html" , "w" ) { |f| f.write(file) }
      do_request "/app.rb/my_form5"
      do_submit  "f" , "var" => "newvalue"
      assert_content "newvalue\n<html><body>newvalue"+
                     "<form name=\"f\" method=\"\"><input type=\"hidden\" name=\"__submitted\" value=\"f\">"+
	    "<input type=\"text\" name=\"var\" value=\"newvalue\"></form></body></html>"
    end

    def test_redirect_to_new_location
	file = "Web::SimpleDispatcher.handle_request( :default => MyForm1 )"
	File.open( temp_dir + "/test_dir/app.rb" , "w" ) { |f| f.write(file) }
	file = "<html><body>{$var}<narf:form name=f>"+
	       "<narf:input type=text name=var></narf:form></body></html>"
	File.open( temp_dir + "/test_dir/templates/my_form6.html" , "w" ) { |f| f.write(file) }

	file = ""
	File.open( temp_dir + "/test_dir/templates/my_form7.html" , "w" ) { |f| f.write(file) }

	do_request "/app.rb/my_form6" 

	do_submit  "f" , "var" => "newvalue"
	assert_content "xxxx"
    end

    def test_display_link
	    file = "Web::SimpleDispatcher.handle_request( :default => MyForm1 )"
	    File.open( temp_dir + "/test_dir/app.rb" , "w" ) { |f| f.write(file) }
      file = "<html><body><narf:link value=\"alink\">Link Text</narf:link></body></html>"
      File.open( temp_dir + "/test_dir/templates/test_link_template.html" , "w" ) { |f| f.write(file) }
      
      do_request( "/app.rb/test_link_template" )
      
      assert_content "<html><body><a href=\"my_form1?a=1&b=2\">Link Text</a></body></html>"
  end
end
