class TestHashCompare < Test::Unit::TestCase
    def test_includes_same
	assert_includes( {"a" => "aa"}  , {"a" => "aa"} )
    end

    def test_includes_missing
	e = assert_raises( Test::Unit::AssertionFailedError ) {
	    assert_includes( { "a" => "aa", "b" => "bb" }  , {"a" => "aa"} )
	}
        assert e.to_s.index( "Missing from actual: b => \"bb\"" )
    end

    def test_includes_different
	e = assert_raises( Test::Unit::AssertionFailedError ) {
	    assert_includes( {"a" => "aa"}   , { "a" => "bb" } )
	}
        assert e.to_s.index("Difference: required <a => \"aa\"> was <a => \"bb\">" )
    end

    def test_includes_multi_level
	e = assert_raises( Test::Unit::AssertionFailedError ) {
	    assert_includes( {"a" => { "a" => "aa", "b" => "bb" }}   , { "a" => { "a" => "aa", "b" => "cc"}} )
	}
        assert e.to_s.index("Difference: required <a.b => \"bb\"> was <a.b => \"cc\">")
    end

    def test_includes_array_with_difference
	e = assert_raises( Test::Unit::AssertionFailedError ) {
	    assert_includes( {"a" => [{ "a" => "aa" }, {"b" => "bb" }]},{ "a" => [ {"a" => "aa"}, {"b" => "cc"}]})
	}
        assert e.to_s.index("Difference: required <a[1].b => \"bb\"> was <a[1].b => \"cc\">")
    end

    def test_includes_array_with_items_missing
	e = assert_raises( Test::Unit::AssertionFailedError ) {
	    assert_includes( { "a" => [ {"a" => "aa"}, {"b" => "cc"}]} , {"a" => [{ "a" => "aa" }]} )
	}
        assert e.to_s.index("Missing from actual: a[1] => {\"b\"=>\"cc\"}")
    end

    def test_includes_hash_when_should_not
	e = assert_raises( Test::Unit::AssertionFailedError ) {
	    assert_includes( {"a" => "b"}  , { "a" => { "a" => "aa", "b" => "cc"}} )
	}
        assert e.to_s.index("Difference: required <a => \"b\"> was <a => {\"a\"=>\"aa\", \"b\"=>\"cc\"}>")
    end

    def test_does_not_include_hash_when_it_should
	e = assert_raises( Test::Unit::AssertionFailedError ) {
	    assert_includes( { "a" => { "a" => "aa" , "b" => "cc"}}, {"a" => "b"}  )
	}
        assert e.to_s.index("Difference: required <a => {\"a\"=>\"aa\", \"b\"=>\"cc\"}> was <a => \"b\">")
    end

    def test_includes_array_when_should_not
	e = assert_raises( Test::Unit::AssertionFailedError ) {
	    assert_includes( {"a" => "a"}, { "a" => [ {"a" => "aa"}, {"b" => "cc"}]} )
	}
        assert e.to_s.index("Difference: required <a => \"a\"> was <a => [{\"a\"=>\"aa\"}, {\"b\"=>\"cc\"}]>")
    end

    def test_does_not_include_array_when_it_should
	e = assert_raises( Test::Unit::AssertionFailedError ) {
	    assert_includes( {"a" => "b"}  , { "a" => { "a" => "aa" , "b" => "cc"}} )
	}
        assert e.to_s.index("Difference: required <a => \"b\"> was <a => {\"a\"=>\"aa\", \"b\"=>\"cc\"}>" )
    end

    def test_assert_includes
        assert_includes( {"a" => { "a" => "aa" }}, { "a" => { "a" => "aa" , "b" => "aa"}} )
    end

    def test_assert_includes2
      assert_raises(Test::Unit::AssertionFailedError) {
        assert_includes( { "a" => { "a" => "aa" , "b" => "cc"}}, {"a" => { "c" => "bb" },"b" => { "c" => "bb" }})
      }
    end
    
    def test_assert_includes_array
        assert_includes( ["a"],[ "a","b"] )
    end
    
    def test_demonstrate_failure
      assert_raises(Test::Unit::AssertionFailedError) {
        assert_includes(["a","b"],[ "a"])
      }
    end
    
    class Trivial
      attr_accessor :name
      def initialize( name )
        @name = name
      end
    end
    
    def test_compare_object_tree_to_hash1
      assert_includes({"name" => "patrick"}, Trivial.new("patrick"))
    end

    def test_compare_object_tree_to_hash2
      assert_includes({"name" => { "name" => "patrick" }}, Trivial.new(Trivial.new("patrick")))
    end

   
    # TODO:
    # reports an error if either item is not a hash
    # improve error reporting when comparing a hash to an array and vice versa
    # add an enhanced version of assert equals
end
