class TestBuffer < Test::Unit::TestCase
  include Web::Testing
  def setup
    Web::setup
  end

  def teardown
    Web::teardown
  end

  #  ob_start -- Turn on output buffering
  def test_ob_start
    Web::open( :out=>'' ) do
      Web::ob_start do |content|
        [ "start", content, "end" ].join(",")
      end
      
      Web::write "middle"
    end
    
    assert_equal( "start,middle,end", Web::get_content )
  end

  #  ob_flush --  Flush (send) the output buffer
  def test_ob_flush
    Web::open( :out=>'' ) do
      Web::ob_start do |content|
        "[" + content +"]"
      end
      
      Web::write "write 1"

      Web::ob_flush

      Web::write "write 2"
    end

    assert_equal( '[write 1][write 2]', Web::get_content )
  end

  #  ob_clean --  erase the output buffer
  def test_ob_clean
    Web::open( :out=>'' ) do 
      Web::write "this text will be erased"
      Web::ob_clean
      Web::write "expected content"
    end
    
    assert_equal( 'expected content', Web::get_content )
  end

  #  ob_end_clean --  Clean (erase) the output buffer and turn off output buffering
  def test_ob_end_clean
    Web::open( :out=>'' ) do
      Web::ob_start do |content|
        "[" + content + "]"
      end
      
      Web::ob_start do |content|
        "{" + content + "}"
      end
      
      Web::write "this content will not be seen"
      
      assert( Web::ob_end_clean )
      
      Web::write "expected content"
    end

    assert_equal( '[expected content]', Web::get_content )
  end
  
  #  ob_end_flush --  Flush (send) the output buffer and turn off output buffering
  def test_ob_end_flush
    Web::open( :out=>'' ) do
      Web::ob_start do |content|
        "[" + content + "]"
      end
      
      Web << "expected content"

      Web::ob_end_flush

      Web << "a little more"
    end

    assert_equal( '[expected content]a little more', Web::get_content )
  end

  #  ob_get_clean --  Get current buffer contents and delete current output buffer
  def test_ob_get_clean
    Web::open( :out=>'' ) do
      Web::ob_start do |content|
        "[" + content + "]"
      end

      Web << 'expected content'
      assert_equal( 'expected content', Web::ob_get_clean )
      Web << 'more content'
    end

    assert_equal( 'more content', Web::get_content )
  end
  
  #  ob_get_contents --  Return the contents of the output buffer
  def test_ob_get_contents
    Web::open( :out=>'' ) do
      Web << 'expected'
      assert_equal( 'expected', Web::ob_get_contents )
    end
    assert_equal( 'expected', Web::get_content )
  end

  #  ob_get_flush --  Flush the output buffer, return it as a string and turn off output buffering
  def test_ob_get_flush
    Web::open( :out=>'' ) do
      Web::ob_start do |content|
        "[" + content + "]"
      end
      Web << 'expected'
      assert_equal( 'expected', Web::ob_get_flush )
      Web::ob_end_clean
    end
    assert_equal( '[expected]', Web::get_content )
  end
  
  #  ob_get_length --  Return the length of the output buffer
  #  ob_get_contents --  Return the contents of the output buffer
  def test_ob_get_length
    Web::open( :out=>'' ) do
      Web << 'expected'
      assert_equal( 8, Web::ob_get_length )
    end
  end
  
  #  ob_get_level --  Return the nesting level of the output buffering mechanism
  def test_ob_get_level
    Web::open( :out=>'' ) do
      assert_equal( 1, Web::ob_get_level )
      Web::ob_start
      assert_equal( 2, Web::ob_get_level )
      Web::ob_start
      assert_equal( 3, Web::ob_get_level )
      Web::ob_end_clean
      assert_equal( 2, Web::ob_get_level )
    end
  end

  #  ob_list_handlers --  List all output handlers in use
  def test_ob_list_handlers
    Web::open( :out=>'' ) do
      Web::ob_start do |content|
        "[" + content + "]"
      end

      assert_equal( "[expected]",
                    Web::ob_list_handlers.last.callback.call('expected') )
    end
  end

  #  ob_get_status --  Get status of output buffers
  #  ob_implicit_flush --  Turn implicit flush on/off
  #  ob_gzhandler --  ob_start callback function to gzip output buffer
  #  output_add_rewrite_var --  Add URL rewriter values
  #  output_reset_rewrite_vars 


end
