class TestCodeLoader < Test::Unit::TestCase
    include Web::Testing
    
    def setup
      @pwd = Dir.pwd
      if File.exists?( temp_dir + "/test_dir/" )
	      rm_rf(temp_dir + "/test_dir")
      end
	    Dir.mkdir(temp_dir + "/test_dir/")
      Dir.chdir(temp_dir + "/test_dir/")
    end
    
    def teardown
      Dir.chdir(@pwd)
	    rm_rf(temp_dir + "/test_dir")
    end
    
    def write_file filename, contents
      File.open( temp_dir + "/test_dir/#{filename}" , "w" ) { |f|
        f.write(contents)
      }
    end

    def test_ruby
      write_file "app.rb", <<-EOF
      print "Hello, World"
      EOF

      do_request "app.rb"

      assert_content "Hello, World"
    end

    def test_phprb
      require 'pp'
      @old_default_handler = Web::config['load_suffix'].default
      Web::config['load_suffix'].default = lambda do |script|
        PHPRB::load(script)
      end
      
      write_file "app.rhtml", <<-EOF
      <?= "Hello, World" ?>
      EOF

      do_request "app.rhtml"

      assert_equal( "Hello, World", Web::get_content.strip )
    ensure
      Web::config['load_suffix'].default = @old_default_handler
    end

    def test_erb
      write_file "app.erb", <<-EOF
      <%= "Hello, World" %>
      EOF
      
      do_request "app.erb"
      
      assert_equal( "Hello, World", Web::get_content.strip )
      
    end

    def test_custom
      Web::config[:load_suffix][:custom] = lambda do |script|
        Web::write script
      end

      write_file "app.custom", <<-EOF
      this code doesn't matter
      EOF
      
      do_request "app.custom"
      assert_match( /app.custom$/, Web::get_content.strip )
    ensure
      Web::config[:load_suffix].delete :custom
    end

end
