class TestErrorHandling < Test::Unit::TestCase
  def app
    temp_dir + "/test_dir/app.rb"
  end

  def setup
    File.delete(app) if File.exists? app
    @stdout, @stderr = $stdout, $stderr
    $stdout, $stderr = StringIO.new, StringIO.new
    Web::setup
  end

  def teardown
    File.delete(app) if File.exists? app
    Web::teardown
    $stdout, $stderr = @stdout, @stderr
  end
  
  def write_file file
    File.open( temp_dir + "/test_dir/app.rb" , "w" ) { |f| f.write(file) }
  end

  def test_production
    Web::config[:error_style]  = :production
    
    Web::open do
      raise "this is a test error"
    end

    assert_equal( '500', Web::status )

    assert_match( /^RuntimeError: this is a test error/,
                  $stderr.string,
                  "Error printed to stderr" )
    assert_match( /<b style='font-size:20px'>this is a test error<\/b>/,
                  $stdout.string,
                  "Error printed with html to stdout" )
    assert_no_match( /test.error.handling.rb/,
                  $stdout.string,
                  "No backtrace should be printed to stdout" )

  end

  def test_development
    Web::config[:error_style] = :development

    Web::open do
      raise "this is a test error"
    end

    assert_equal( '500', Web::status )

    assert_match( /^RuntimeError: this is a test error/,
                  $stderr.string,
                  "Error printed to stderr" )
    assert_match( /<b style='font-size:20px'>this is a test error<\/b>/,
                  $stdout.string,
                  "Error printed with html to stdout" )
    assert_match( /test.error.handling.rb/,
                  $stdout.string,
                  "backtrace printed" )
  end
  
  def test_content_type
    Web::config[:error_style] = :development
    
    Web::open do
      Web::content_type = "application/ms-excel"
      raise "this is a test error"
    end
    
    assert_match( /Content-Type: text\/html/,
                  $stdout.string,
                  "Content-Type Changed back to text/html" )
  end

    
    

end
