class TestLoad < Test::Unit::TestCase
  def cleanup
    if File.exists?( make_temp( "script.narf" ) )
      File.delete( make_temp( "script.narf" ) ) 
    end
  end

  def setup
    cleanup
    Web::connection = Web::Connection::new( :out => "" )
  end

  def teardown
    cleanup
  end
  
  def script( src )
    File.open( make_temp( "script.narf" ), 'w' ) do |f|
      f.write src
    end
  end

  def run_script
    Web::open(:out => '') do
      Web::load( make_temp( "script.narf" ) )
    end
  end

  def test_hello_world1
    script <<-SCRIPT
Hello World
SCRIPT
    run_script
    assert_equal( "Hello World\n", Web::get_content )
  end

  def test_hello_world2
    script <<-SCRIPT
<%= "Hello World" %>
SCRIPT
    run_script
    assert_equal( "Hello World\n", Web::get_content )
  end

  def test_hello_world3
    script <<-SCRIPT
<% print "Hello World" %>
SCRIPT
    run_script
    assert_equal( "Hello World\n", Web::get_content )
  end
  
  def test_hello_world4
    script <<-SCRIPT
<%= "Hello World" %>
SCRIPT
    run_script
    assert_equal( "Hello World\n", Web::get_content )
  end

  def test_hello_world5
    script <<-SCRIPT
<% print "Hello World" %>
SCRIPT
    run_script
    assert_equal( "Hello World\n", Web::get_content )
  end

  def test_hello_world6
    script <<-SCRIPT
<% def print_hello
    print "Hello World"
   end
   print_hello
   %>
SCRIPT
    run_script
    assert_equal( "Hello World\n", Web::get_content )
  end

end

class TestApplication < Test::Unit::TestCase
  include Web::Testing
  
  def setup
    teardown
    Web::docroot = make_temp
    Web::setup
  end

  def teardown
    Web::teardown
    rm_rf make_temp
  end
  
  def test_application_rb
    write_file('application.rb', <<-APP )
    Web::puts "application"
    APP

    write_file('script.rb', <<-SCRIPT )
    Web::puts "script"
    SCRIPT

    do_request('script.rb')
    assert_content( "application\nscript\n" )
  end

  def test_application_in_parent_dir
    write_file('application.rb', <<-APP )
    Web::puts "application"
    APP

    write_file('subdir/script.rb', <<-SCRIPT )
    Web::puts "subdir/script"
    SCRIPT
    
    #Web::load_application( 'subdir/script.rb' )
               
    do_request('subdir/script.rb')
    assert_content( "application\nsubdir/script\n" )
  end
end

