class TestWebCgi < Test::Unit::TestCase
  def set_up
    teardown
  end

  def teardown
    $stderr = $deferr #@old_stderr
    @cgi = @options = @mockio = @params = @cookie = nil
  end
  
  def output
    @mockio ||= ""
  end
  
  def params
    @params ||= { "field"  => "value"   }
  end
  
  def cookie
    @cookie ||= { "cookie" => "monster" }
  end
  
  def options
    @options ||= {:out     => output,
                  :request => params,
                  :cookie  => cookie }
  end
  
  def cgi
    @cgi ||= Web::Connection.new options
  end
  
  def test_initialize
    assert_equal( output,  cgi.output        )
    assert_equal( params,  cgi.request )
    assert_equal( Web::Connection::normalize(cookie), cgi.cookie         )
  end

    def test_output
      cgi << "<h1>Hello</h1>"
      cgi.flush
      assert_match( /Content-Type: text\/html\r\n/, output )
      assert_match( /\r\n<h1>Hello<\/h1>/,          output )
    end
    
    def test_get_content
      # test blank result
      assert_equal( "", cgi.get_content )
      
      # test content
      cgi << "<h1>hola mundo!</h1>"
  cgi.close
	assert_equal( "<h1>hola mundo!</h1>", cgi.get_content )

	# test return value, not reference
	content = cgi.get_content 
	content += " some trailing garbage"
	assert_equal( "<h1>hola mundo!</h1>", cgi.get_content )
    end

    def test_assert_content
        assert_raises(Test::Unit::AssertionFailedError){
	    cgi.assert_content "No content here"
	}
	cgi.write "Here is some content"
      cgi.flush
	cgi.assert_content "Here is some content"
    end

    # this is a bit incomplete -- only encodes ampersands
    def test_html_encode
      assert_equal( '&amp;', Web::html_encode( '&' ) )
    end

    def test_escape
	assert_equal( '%21%40%24%25%5E%26%2A%28%29', Web::escape('!@$%^&*()') )
    end

    def test_unescape
	assert_equal( '!@$%^&*()', Web::unescape('%21%40%24%25%5E%26%2A%28%29') )
    end

    def test_rfc1123_date
	assert_match( /[A-Z][a-z]{2}, \d{1,2} [A-Z][a-z]{2} \d{4} \d\d:\d\d:\d\d GMT/,
		      Web::rfc1123_date(Time.now) )
    end

    def test_web_process_returns_the_cgi
      cgi = Web::open( :out => "" ){ |cgi|
        cgi.write "Hello World"
      }
      
      assert_equal( "Hello World", cgi.get_content )
    end

    # instead of one instance per output stream,
    # how about one instance per thread
    # this is tested by testing installation under webrick
    def deprecated_test_get_instance_of_cgi_per_output_stream
	out = ""
	assert_equal Web::Connection.create( :out => out ), Web::Connection.create( :out => out )
	assert_not_equal Web::Connection.create( :out => "" ), Web::Connection.create( :out => "" )
    end
end


