class TestSession < Test::Unit::TestCase
  def test_session_ids_are_uniq
    assert_equal( 1000, Array.new( 1000 ).collect {
    Web::Session.create_id
  }.uniq.length )
  end

  def test_set_value_on_session
    cgi = Web::Connection.new( :out => "" )
    cgi.session["testvar"] = "test"
    assert_equal( "test", cgi.session["testvar"] )
  end

  def test_session_end_to_end
    cgi = Web::Connection.new( :out => "" )
    cgi.session["key"] = ["value"]
    session_id = cgi.cookies_sent["_session_id"]

    Web::Connection.new( :out => "", :cookies => { "_session_id", session_id } )
    
    assert_equal( ["value"], cgi.session["key"] )
  end

  def test_clear_session
    cgi = Web::Connection.new( :out => "" )
    cgi.session["aKey"] = "aValue"
    cgi.close

    session_id = cgi.cookies_sent["_session_id"]
    Web::Connection.new( :out => "", :cookies => { "_session_id", session_id } )

    cgi.session.reset
    assert_nil( cgi.session["aKey"] )
  end
  
  def test_can_iterate_over_session
    cgi = Web::Connection.new( :out => "" )
    cgi.session["aKey"] = "1"
    cgi.session["aKey2"] = "2"
    cgi.close

    values = []
    cgi.session.each do |i,j|
	    values << j
    end
    assert_equal [ "1" , "2" ], values
  end
end


