require 'web/shim/cgi'
    
class TestCGIShim < Test::Unit::TestCase
  include Web::Testing
  
  def setup
    @pwd = Dir.pwd
    if File.exists?( temp_dir + "/test_dir/" )
      rm_rf(temp_dir + "/test_dir")
    end
    Dir.mkdir(temp_dir + "/test_dir/")
    Dir.chdir(temp_dir + "/test_dir/")
    #@docroot = Web.get_docroot
    #Web.set_docroot( temp_dir + "/test_dir/")
  end
    
  def teardown
    Dir.chdir(@pwd)
    rm_rf(temp_dir + "/test_dir")
    #Web.set_docroot( @docroot )
  end
  
  def write_file file
    File.open( temp_dir + "/test_dir/app.rb" , "w" ) do |f|
      f.write <<-EOF
      require 'web/shim/cgi'
      cgi = CGI.new
      require 'test/unit/assertions'
      include Test::Unit::Assertions
      EOF
      f.write file
    end
  end

  def test_single_params
    write_file <<-EOF
    assert_equal( 'Dave Thomas', cgi['name']   )
    assert_equal( 'flexible',    cgi['reason'] )
    EOF
    
    do_request( "app.rb",
                'name' => 'Dave Thomas',
                'reason' => 'flexible' )
  end

  def test_multiple_params
    write_file <<-EOF
    assert_equal( ['Dave Thomas'], cgi.params['name']   )
    assert_equal( ['flexible',
                   'transparent',
                   'fun'],
                  cgi.params['reason'] )
    cgi.params['name'] = [ cgi['name'].upcase ]
    assert_equal( {'name' => ['DAVE THOMAS'],
                   'reason' => ['flexible','transparent','fun'] },
                  cgi.params )
    EOF
    
    do_request( "app.rb",
                'name' => 'Dave Thomas',
                'reason' => ['flexible','transparent','fun'] )

  end

  def test_has_key?
    write_file <<-EOF
    assert_equal( true,  cgi.has_key?('name') )
    assert_equal( false, cgi.has_key?('age')  )
    EOF
    
    do_request( "app.rb",
                'name' => 'Dave Thomas',
                'reason' => ['flexible','transparent','fun'] )
  end
  
  def test_escape
    str = "Nicholas Payton/Trumpet & Flugel Horn"
    esc = 'Nicholas+Payton%2FTrumpet+%26+Flugel+Horn'
    assert_equal(esc, CGI.escape(str)   )
    assert_equal(str, CGI.unescape(str) )
  end

  def test_escapeHTML
    str = 'a < 100 && b > 200'
    esc = 'a &lt; 100 &amp;&amp; b &gt; 200'
    assert_equal(esc, CGI.escapeHTML(str) )
    assert_equal(str, CGI.unescapeHTML(esc) )
  end

  def test_escapeElement
    str = '<hr><a href="/mp3">Click Here</a><br>'
    esc = '<hr>&lt;a href=&quot;/mp3&quot;&gt;Click Here&lt;/a&gt;<br>'
    assert_equal( esc, CGI::escapeElement( str, 'A' ) )
    assert_equal( str, CGI::unescapeElement( str, 'A' ) )
  end
end
