class TestAttributeParser < Test::Unit::TestCase
    def test_test_no_attributes
	block_used = false
	StrScanParser.new(StringScanner.new("<narf:name>")).match_tag { |tag|
	    assert_equal( "name", tag.name )
	    assert_equal( {}, tag.attributes )
	    block_used = true
	}
	assert block_used
    end

    def test_test_one_attribute
	block_used = false
	StrScanParser.new(StringScanner.new("<narf:name foo=bar>")).match_tag { |tag|
	    assert_equal( "name", tag.name )
	    assert_equal( { "foo" => "bar" }, tag.attributes )
	    block_used = true
	}
	assert block_used
    end

    def test_quoted_value
	block_used = false
	StrScanParser.new(StringScanner.new("<narf:name foo=\"bar x\">")).match_tag{ |tag|
	    assert_equal( "name", tag.name )
	    assert_equal( { "foo" => "bar x" }, tag.attributes )
	    block_used = true
	}
	assert block_used
    end

    def test_test_two_attributes
	block_used = false
	StrScanParser.new(StringScanner.new("<narf:name foo=bar x=y>")).match_tag{|tag|
	    assert_equal( "name", tag.name )
	    assert_equal( { "foo" => "bar", "x" => "y" }, tag.attributes )
	    block_used = true
	}
	assert block_used
    end

    def test_test_unclosed_tag
	assert_raises(TagNotClosedException) {
	    StrScanParser.new(StringScanner.new("<narf:name foo=bar x=y")).match_tag {}
	}
    end

    def test_test_no_tag_name
	assert_raises(TagNotNamedException) {
	    StrScanParser.new(StringScanner.new("<narf: name foo=bar x=y")).match_tag  {}
	}
    end

    def test_missing_attribute_value
	assert_raises(MissingAttributeException) {
	    StrScanParser.new(StringScanner.new("<narf:name foo=")).match_tag  {}
	}
    end

    def test_missing_quote
	assert_raises(MissingQuoteException) {
	    StrScanParser.new(StringScanner.new("<narf:name foo=\"foo}")).match_tag  {}
	}
    end
end
