class TestWebTesting2 < Test::Unit::TestCase
    include Web::Testing
    
    def setup
      @pwd = Dir.pwd
      if File.exists?( temp_dir + "/test_dir/" )
	      rm_rf(temp_dir + "/test_dir")
      end
	    Dir.mkdir(temp_dir + "/test_dir/")
      Dir.chdir(temp_dir + "/test_dir/")
      #@docroot = Web.get_docroot
      #Web.set_docroot( temp_dir + "/test_dir/")
    end
    
    def teardown
      Dir.chdir(@pwd)
	    rm_rf(temp_dir + "/test_dir")
	    #Web.set_docroot( @docroot )
    end
    
    def write_file file
      File.open( temp_dir + "/test_dir/app.rb" , "w" ) { |f| f.write(file) }
    end

    def test_web_cgi_asserts
	write_file <<-EOF
	Web.print "hello tom"
	Web.set_cookie( "username", "someonespecial" )
	Web.add_header( "Special-Person", "Oh Yes" )
	EOF

        do_request "app.rb"

        assert_content "hello tom"
        assert_header "Special-Person", "Oh Yes"
        assert_cookie "username", "someonespecial"
    end

    def test_path_info
      write_file <<-EOF
      Web::write Web::path_info
      EOF
      do_request "app.rb/path_info/"
      assert_content( "/path_info/" )
    end
    
    def test_do_submit
        form = "<html><body><form name='the_form'><input type='text' name='thing'></form></body></html>"

	write_file <<-EOF
	if Web["thing"] == "foo"
            Web << "this is a foo page"
	elsif Web["thing"] == "bar"
	    Web << "that is the bar page"
	end
	Web << "#{form}"
        EOF

        do_request "app.rb"
        assert_content( form )
        
        do_submit( "the_form", "thing" => "foo" )
        
        assert_content( "this is a foo page" + form )

        do_submit( "the_form", "thing" => "bar" )

        assert_content( "that is the bar page" + form )
    end

    def test_do_submit_submits_hidden
        form = "<html><body><form name='the_form'><input type='hidden' name='thing' value='val'></form></body></html>"

	write_file <<-EOF
	Web << "displayed '"
	Web << Web['thing']
	Web << "'#{form}"
        EOF

        do_request "app.rb"
        assert_content( "displayed ''" + form)

        do_submit( "the_form" )
        
        assert_content( "displayed 'val'" + form)
    end
  
    def test_do_redirect
	write_file <<-EOF
	if Web["action"] == "finish"
	    Web.puts "We redirected!"
	else
	    Web.set_redirect "app.rb?action=finish" 
	    Web.puts "here"
	end
	EOF
	
        do_request "app.rb"

	assert_content( "We redirected!\n" )
    end

    def test_do_redirect_with_session
	write_file <<-EOF
	if Web["action"] == "finish"
	    Web.puts Web.session["name"]
	else	
	    Web.set_redirect "app.rb?action=finish" 
	    Web.session["name"] = "Tom"
	end
	EOF

        do_request( "app.rb" )

	assert_content( "Tom\n" )
    end


    def test_do_multiple_redirect
	write_file <<-EOF
	if Web["action"] == "second"
	    Web.set_redirect "app.rb?action=third"
	elsif Web["action"] == "third"
	    Web.puts "We redirected!"
	else
	    Web.set_redirect "app.rb?action=second" 
	end
	EOF

        do_request "app.rb"

	assert_content( "We redirected!\n" )
    end

    def test_do_redirect_on_submit
	write_file <<-EOF
	if Web["submitted"] == "true"
	    Web.set_redirect "app.rb?isredirect=true"
	elsif Web["isredirect"] == "true"
	    Web.puts "We redirected!"
	else
	    Web.puts "<html><body><form name='form' action='app.rb'>"
	    Web.puts "<input type='hidden' name='submitted' value='true'>"
	    Web.puts "</form></body></html>"
	end
	EOF

  do_request "app.rb"
      do_submit "form"

      assert_content( "We redirected!\n" )
    end
    
    def test_form_exists
	write_file <<-EOF
	Web.puts "<html><body><form name='form' action='app.rb'>"
	Web.puts "<input type='hidden' name='submitted' value='true'>"
	Web.puts "</form></body></html>"
	EOF

  do_request "app.rb"
      assert_raises( FormNotFoundException ) do
	    do_submit "not_this_form"
	end
    end

    def test_all_fields_on_form
	write_file <<-EOF
	Web.puts "<html><body><form name='form' action='app.rb'>"
	Web.puts "</form></body></html>"
	EOF

    do_request "app.rb"
	  assert_raises( FieldNotFoundException ) do
	    do_submit "form", "not_on_form" => "value"
	end
    end

    # check that sessions & cookies are propagated

    def test_two_hidden_forms_are_array
        form = "<html><body><form name=\"the_form\">"+
	    "<input type=\"hidden\" name=\"thing[]\" value=\"val1\">"+
	    "<input type=\"hidden\" name=\"thing[]\" value=\"val2\">"+
	    "</form></body></html>"

	write_file <<-EOF
	Web << "submitted '"
	Web << Web['thing[]'].sort.join(',')
	Web << "'"
	Web << '#{form}'
        EOF

        do_request "app.rb"
        assert_content( "submitted ''" + form)

        do_submit( "the_form" )
        
        assert_content( "submitted 'val1,val2'" + form)
    end


    def test_cgi_optional
	    write_file <<-EOF
	      if Web["name"] == "patrick"
          Web.write( "temp_patrick" )
        elsif Web["name"] == "tom"
	        Web << "My name is "
	        Web << Web['name']
	        Web::puts " and also TOM"
	      end
	    EOF
      
      do_request( "app.rb" , "name" => "patrick" )
      assert_content "temp_patrick"
      
      do_request( "app.rb", "name" => "tom" )
      assert_content "My name is tom and also TOM\n"
    end

    def test_assert_form_includes_input_box
	write_file <<-EOF
	    Web.puts "<html><body><form name='form'>"+
		"<input type='text' name='test1' value='1'>"+
		"<input type='text' name='test2' value='2'>"+
		"</form></body></html>" 
	EOF
	do_request "app.rb"

	assert_form_includes("form" , { "test1" => "1", "test2" => "2" } )
	assert_form_includes("form" , { "test1" => "1" } )	    
	
	assert_raises( Test::Unit::AssertionFailedError ) {
	    assert_form_includes("form" , { "test1" => "1", "test2" => "2", "test3" => "3" } )
	}
	assert_raises( Test::Unit::AssertionFailedError ) {
	    assert_form_includes("form" , { "test1" => "1", "test2" => "1" } )	    
	}
    end
 
    def test_assert_form_includes_checkbox
	write_file <<-EOF
	Web.puts "<html><body><form name='form'>
	          <input type='checkbox' name='test1' checked>
	          <input type='checkbox' name='test2'>
	          </form></body></html>" 
	EOF

	do_request "app.rb"

	assert_form_includes("form" , { "test1" => true, "test2" => false } )
    end

    def todo_test_assert_form_includes_checkbox_array
	write_file <<-EOF
	Web.puts "<html><body><form name='form'>"+
	    "<input type='checkbox' name='test22[]' value='2' checked>"+
	    "<input type='checkbox' name='test22[]' value='3' checked>"+
	    "</form></body></html>" 
	EOF
	do_request "app.rb"
	
	assert_form_includes("form" , { "test22[]" => ["2","3"] } )
    end

    def todo_test_submit_checkbox
	write_file <<-EOF
	Web.puts "<html><body><form name='form'>"+
	    "<input type='hidden' name='action' value='submit'>"+
	    "<input type='checkbox' name='test1' value='1'>"+
	    "<input type='checkbox' name='test2' value='2'>"+
	    "<input type='checkbox' name='test3' value='3' checked>"+
	    "<input type='checkbox' name='test4[]' value='4'>"+
	    "<input type='checkbox' name='test4[]' value='5'>"+
	    "<input type='checkbox' name='test4[]' value='6' checked>"+
	    "</form></body></html>" 
	EOF

	do_request "app.rb"

	actionPerformed = false
	write_file <<-EOF
	assert_equal "", Web["test1"]
	assert_equal "2", Web["test2"]
	assert_equal "3", Web["test3"]	
	assert_equal ["5","6"], Web["test4[]"].sort	
	actionPerformed=true
	EOF

	do_submit( "form", "test2" => "2", "test4[]" => "5" )
	
	assert actionPerformed
    end
    
    def test_assert_form_includes_input_textarea1
	write_file <<-EOF
	Web.puts "<html><body><form name='form'>"+
	    "<textarea name='test1'>contents of text area</textarea>"+
	    "</form></body></html>" 
	EOF
	
	do_request "app.rb"

	assert_form_includes "form" , "test1" => "contents of text area" 
    end   

    def test_assert_form_includes_input_textarea2
	write_file <<-EOF
	Web::puts "<html><body><form name='form'>"+
	    "<textarea name='test1'>contents of text area</textarea>"+
	    "</form></body></html>" 
	EOF
	
	do_request "app.rb"

	assert_form_includes "form", "test1" => "contents of text area"
    end   
end
