class TestIllustrationPages < Test::Unit::TestCase
  include Web::Testing
  
  def setup
    setup_wiki
  end
  def teardown
    teardown_wiki
  end

  def test_nil_illustration
      page = Web::Wiki::Page.new("somepage")
      page.illustration = nil
      assert_equal( "basic.html", page.template )
  end
  
  def delete_test_save_illustration_options
    name = "ThingPage"
    do_request "wiki.rb", "action" => "edit", "page.name" => name
    Web::local.assert_includes(
			 "page" => {
			   "name" => name,
			   "content" => "",
                           "illustration" => "",
			   "text_color"  => "000000",
			   "bg_color"    => "ffffff",
			   "valign"      => "top",
			   "align"       => "left",
			   "top_margin"  => "5",
			   "left_margin" => "5",
			 },
			 "options" => {
			   "align"      => ["left", "center", "right"],
			   "valign"     => ["top",  "middle", "bottom"]
			 }
			 )
			 
			   
    page_hash = {
      "content"     => "Hello Patrick",
      "illustration" => "image.gif",
      "text_color"  => "0000ff",
      "bg_color"    => "cccc00",
      "valign"      => "middle",
      "align"       => "center",
      "top_margin"  => "5",
      "left_margin" => "7",
    }

    do_submit "edit_page", "page" => page_hash, "action" => "Save"
    
    assert_template_used( "illustration.html" )
    Web::local.assert_includes "page" => page_hash
    
  end

  def deleted_test_page_type
    name = "ThingPage"
    do_request "wiki.rb", "action" => "edit", "page.name" => name
    Web::local.assert_includes("page" => {"name" => name,
                                    "content" => "",
                                    "illustration" => "",} )
    
    page_hash = {
      "content"     => "Hello Patrick",
      "illustration" => "http://www.mysite.com/animage.gif",
    }

    do_submit "edit_page", "page" => page_hash, "action" => "Save"
    
    assert_template_used( "illustration.html" )

    Web::local.assert_includes "page" => page_hash
    
  end

  def test_select_illustration
    do_request "wiki.rb", "action" => "select_illustration", "page.name" => name
    Web::local.assert_includes("page" => {"name" => name,},
                         "action" => "select_illustration" )
    
    assert_template_used( "images.html" )
  end

  def do_upload( filename, calling_action = "images", upload = "test_upload.gif" )
    File.copy( "test/wikitestroot/" + upload, "test/wikitestroot/temp.gif" )
      do_request("wiki.rb",
		 "action" => "Upload",
		 "page.name" => "HomePage",
                 "calling_action" => calling_action,
		 "upload" => Web::Upload.new( "test/wikitestroot/temp.gif", "images/gif", filename ) )
    File.delete( "test/wikitestroot/temp.gif" ) if File.exists? "test/wikitestroot/temp.gif"

  end

  def test_upload_image
    do_request "wiki.rb", "action" => "images", "page.name" => "HomePage"
    Web::local.assert_includes("page" => {"assets" => []} )
    assert_form_includes("images",
                         "action" => "Upload",
                         "calling_action" => "images" )
    
    do_upload( "my_image_file.gif" )
    
    assert_template_used( "images.html" )
    assert_template_not_used( "template.html" )

    # assert that HomePage has an image, and that action has been restored
    Web::local.assert_includes("page" => {
			   "name" => "HomePage",
			   "assets" => ["my_image_file.gif"]
			 },
    "action" => "images" )

    # assert our upload form is back and in order
    assert_form_includes("images",
                         "action" => "Upload",
                         "calling_action" => "images" )
    
    # assert that HomePage's images exist in a particular place
    assert File.exists?( Web::Wiki.store_dir + "/HomePage/my_image_file.gif" )
    
    # assert images are the same
    assert_equal( open( Web::Wiki.store_dir + "/HomePage/my_image_file.gif", "r" ) { |f|
                    f.read
		  },
		  open( "test/wikitestroot/test_upload.gif", "r" ) { |f|
		    f.read
    		  } )
  end

  def test_upload_two_images
    do_upload( "first.gif" )
    do_upload( "second.gif" )

    Web::local.assert_includes("page" => {
			   "name" => "HomePage",
			   "assets" => ["first.gif", "second.gif"]
			 } )
  end

  def test_delete_image
    do_upload( "first.gif" )
    do_upload( "second.gif" )

    do_request("wiki.rb",
	       "action" => "delete_asset",
	       "page.name" => "HomePage",
	       "asset" => "first.gif",
               "calling_action" => "images" )
    
    assert_template_used( "images.html" )
    assert_template_not_used( "template.html" )

    Web::local.assert_includes("page" => {
			   "name" => "HomePage",
			   "assets" => ["second.gif"],
			   "historical_assets" => {
			     "first.gif"  => ["\#deleted.first.gif"],
			     "second.gif" => ["second.gif" ] }
			 },
                         "action" => "images")
  end

  def test_delete_same_image
    do_upload( "image.gif" )
    do_upload( "image.gif" )
    do_upload( "image.gif" )
    Web::local.assert_includes( "page" => {
			    "assets" => [ "image.gif" ],
    			    "historical_assets" => {
			      "image.gif" => [ "\#1.image.gif",
					       "\#2.image.gif",
			    		       "image.gif" ] } } )
  end

  def test_upload_with_full_paths
    do_upload( "c:\\Documents and Settings\\Patrick\\Desktop\\first.gif" )
    do_upload( "/home/patsplat/.Desktop/pretty pictures/second.gif" )

    Web::local.assert_includes("page" => {
			   "name" => "HomePage",
			   "assets" => ["first.gif", "second.gif"]
			 } )
  end

  def test_download
    do_upload( "c:\\Documents and Settings\\Patrick\\Desktop\\first.gif" )

    Web::local.assert_includes("page" => {
			   "name" => "HomePage",
			   "assets" => ["first.gif"],
			   "download_link" => "../test.pages/HomePage/"
			 } )
    
    do_request( "wiki.rb",
	        "page.name" => "HomePage",
	        "action" => "download",
	        "asset" => "first.gif" )
    
    expected_asset = nil
    File.open( "test/wikitestroot/test_upload.gif", "r" ) { |f|
      expected_asset = f.read
    }

    assert_equal( expected_asset, Web.connection.get_content )
    assert_equal( ["image/gif"], Web.connection.get_header("Content-Type") )
  end

  def test_view_historical_assets
    do_upload( "first.gif" )
    do_upload( "first.gif" )
    do_upload( "first.gif" )

    do_request( "wiki.rb",
	        "page.name" => "HomePage",
	        "action" => "asset_history",
	        "asset" => "first.gif" )

    Web::local.assert_includes( "page" => {"name" => "HomePage"},
                          "asset" => {"name" => "first.gif",
			              "history" => ["\#1.first.gif","\#2.first.gif","first.gif",]  } )
    assert_template_used( "asset_history.html" )
  end
  
  def test_remove_absolute_download_links
    do_upload( "c:\\Documents and Settings\\Patrick\\Desktop\\first.gif" )
    do_request( "wiki.rb",
                "action" => "Save",
	        "page.name" => "HomePage",
	        "page.content" => "<img src=\"http://mysite.com/cgi-bin/../test.pages/HomePage/first.gif\" border=0>" )

    Web::local.assert_includes( "page" => {
			   "name" => "HomePage",
			   "content" => "<img src=\"../test.pages/HomePage/first.gif\" border=0>",
			 } )

    #do_request( "wiki.rb",
    #            "action" => "Save",
#	        "page.name" => "HomePage",
#	        "page.content" => "<img src=\"http://mysite.com/cgi-bin/wiki.rb?action=download&amp;page.name=HomePage&amp;asset=first.gif\" border=0>" )

#    Web::local.assert_includes( "page" => {
#			   "name" => "HomePage",
#			   "content" => "<img src=\"wiki.rb?action=download&amp;page.name=HomePage&amp;asset=first.gif\" border=0>",
#			 } )
  end

  def test_insert_download
    do_request("wiki.rb",
               "action" => "insert_download",
               "page.name" => "HomePage" )
    Web::local.assert_includes("page" => {"assets" => []} )
    assert_form_includes("images",
			 "action" => "Upload",
                         "calling_action" => "insert_download" )
    
    do_upload( "my_image_file.gif", "insert_download" )

    assert_template_used( "images.html" )
    assert_template_not_used( "template.html" )

    # assert that HomePage has an image, and that action has been restored
    Web::local.assert_includes("page" => {
			   "name" => "HomePage",
			   "assets" => ["my_image_file.gif"]
			 },
                         "action" => "insert_download" )

    # assert our upload form is back and in order
    assert_form_includes("images",
			 "action" => "Upload",
                         "calling_action" => "insert_download" )
  end

  def test_page_assets
    do_upload( "small.gif" )
    do_upload( "large.jpg", "images", "big_image.jpg" )
    page = Web::Wiki::wiki.load_page( "HomePage" )
    expected = ["large.jpg (<i>114452 bytes</i>)",
                "<img src='../test.pages/HomePage/small.gif' border=0 width=100><br>small.gif" ]
    assert_equal( expected, page.assets.collect{ |e| e.thumbnail } )
  end


end

