require 'web/wiki'

class TestLinker< Test::Unit::TestCase
  include Web::Testing

  def set_content( page, content ) 
    do_request "wiki.rb", "action" => "edit", "page.name" => page
    do_submit "edit_page", "page.content" => content, "action" => "Save"
    do_request "wiki.rb", "page.name" => page
  end

  def setup
    setup_wiki
  end

  def teardown
    teardown_wiki
  end
  
  def test_make_edit_link
      do_request "wiki.rb", "action" => "edit", "page.name" => "HomePage"
      do_submit "edit_page", "page.content" => "HelloPatrick", "action" => "Save"
      do_request "wiki.rb", "page.name" => "HomePage"

      Web::local.assert_includes "page" => {
	"name" => "HomePage",
	"html" => "HelloPatrick<a href=\"?page.name=HelloPatrick&action=edit\">?</a>"
      }
  end

  def test_make_edit_link_with_spaces
      do_request "wiki.rb", "action" => "edit", "page.name" => "HomePage"
      do_submit "edit_page", "page.content" => "[[Hello Patrick]]", "action" => "Save"
      do_request "wiki.rb", "page.name" => "HomePage"

      Web::local.assert_includes "page" => {
	"name" => "HomePage",
	"html" => "Hello Patrick<a href=\"?page.name=Hello+Patrick&action=edit\">?</a>"
      }
  end

  #def test_link_substrings_and_longer_links
  #  set_content( "HomePage",
  #               "This is a test of linking SubString and SubStringPage correctly" )

#    Web::local.assert_includes "page" => {
#	"name" => "HomePage",
#	"html" => "This is a test of linking SubString<a href=\"?page.name=SubString&action=edit\">?</a> and SubStringPage<a href=\"?page.name=SubStringPage&action=edit\">?</a> correctly"
#      }


 #   set_content( "SubString", "" )
 #   set_content( "SubStringPage", "")
    
 #   set_content( "HomePage",
 #                "This is a test of linking SubString and SubStringPage correctly" )

  #    Web::local.assert_includes "page" => {
#	"name" => "HomePage",
#	"html" => "This is a test of linking <a href=\"?page.name=SubString\">SubString</a> and <a href=\"?page.name=SubStringPage\">SubStringPage</a> correctly"
#      }
#  end

  def test_excluding_anchor_tags
      set_content("HomePage",
		  "This is a Test of a <a href=Test>Test</a> that should Test fine" )
      
      Web::local.assert_includes "page" => {
	"name" => "HomePage",
	"html" => "This is a Test of a <a href=Test>Test</a> that should Test fine"
      }
      
      set_content( "SomePage", "test" )
      set_content( "DorkPage", "test" )
      set_content( "HomePage",
		  "This page links to SomePage and the DorkPage and a NonExistentPage with an extra <a href=/FrontPage>FrontPage</a> link thrown in for fun." )
      
      Web::local.assert_includes "page" => {
	"name" => "HomePage",
	"html" => "This page links to <a href=\"?page.name=SomePage\">SomePage</a> and the <a href=\"?page.name=DorkPage\">DorkPage</a> and a NonExistentPage<a href=\"?page.name=NonExistentPage&action=edit\">?</a> with an extra <a href=/FrontPage>FrontPage</a> link thrown in for fun."
      }
  end

  def test_exclude_tags
      set_content("HomePage",
		  "<a href=SomePage><form action='SomePage'><input value=\"SomePage\">" )
      do_request( "wiki.rb", "page.name" => "HomePage" )
      Web::local.assert_includes "page" => {
	"name" => "HomePage",
	"html" => "<a href=SomePage><form action='SomePage'><input value=\"SomePage\">"
      }
  end

  def test_link_slashed_words
    do_request "wiki.rb", "action" => "edit", "page.name" => "HomePage"
    do_submit "edit_page", "page.content" => "Hello/Patrick", "action" => "Save"
    do_request "wiki.rb", "page.name" => "HomePage"
    
    Web::local.assert_includes "page" => {
      "name" => "HomePage",
      "html" => "Hello/Patrick<a href=\"?page.name=Hello%2FPatrick&action=edit\">?</a>"
    }
    
    do_request("wiki.rb",
               "page.name" => "Hello/Patrick",
               "action" => "Save",
	       "content" => "hola")

    do_request "wiki.rb", "page.name" => "HomePage"
    
    Web::local.assert_includes "page" => {
      "name" => "HomePage",
      "html" => "<a href=\"?page.name=Hello%2FPatrick\">Hello/Patrick</a>",
    }

    do_request "wiki.rb", "page.name" => "Hello/Patrick"
    Web::local.assert_includes "page" => {
      "name" => "Hello/Patrick",
      "download_link" => "../test.pages/Hello-slash-Patrick/",
    }
  end

  def test_make_edit_link_with_bumpy_and_spaces
      do_request "wiki.rb", "action" => "edit", "page.name" => "HomePage"
      do_submit "edit_page", "page.content" => "[[Hello PatrickMay]]", "action" => "Save"
      do_request "wiki.rb", "page.name" => "HomePage"

      Web::local.assert_includes "page" => {
	"name" => "HomePage",
	"html" => "Hello PatrickMay<a href=\"?page.name=Hello+PatrickMay&action=edit\">?</a>"
      }
  end  

  def test_make_edit_link_with_bumpy_in_word
      do_request "wiki.rb", "action" => "edit", "page.name" => "HomePage"
      do_submit "edit_page", "page.content" => "helloPatrickMay", "action" => "Save"
      do_request "wiki.rb", "page.name" => "HomePage"

      Web::local.assert_includes "page" => {
	"name" => "HomePage",
	"html" => "helloPatrickMay"
      }
  end

  def test_make_path_info_link
    set_content( "SomePage", "make this page exist" )
    set_content( "HomePage", "link to SomePage" )
    do_request "wiki.rb", "page.name" => "HomePage"
    
    Web::local.assert_includes "page" => {
      "name" => "HomePage",
      "html" => "link to <a href=\"?page.name=SomePage\">SomePage</a>"
    }

    Web::Wiki::set_pref( :script_url => "/cgi-bin/wiki.rb",
                         :store_url  => nil )

    Web::local.assert_includes "page" => {
      "name" => "HomePage",
      "html" => "link to <a href=\"?page.name=SomePage\">SomePage</a>"
    }

    Web::Wiki::set_pref( :script_url => "/cgi-bin/wiki.rb",
                         :store_url  => "/pages/")

    Web::local.assert_includes "page" => {
      "name" => "HomePage",
      "html" => "link to <a href=\"/cgi-bin/wiki.rb/SomePage.html\">SomePage</a>"
    }

    do_request( "wiki.rb/SomePage.html" )
    
    Web::local.assert_includes "page" => {
      "name" => "SomePage",
      "html" => "make this page exist"
    }

    set_content( "SomePage/SubPage", "make this exist" )
    set_content( "SomePage", "link to SomePage/SubPage" )

    do_request( "wiki.rb/SomePage.html" )

    Web::local.assert_includes "page" => {
      "name" => "SomePage",
      "html" => "link to <a href=\"/cgi-bin/wiki.rb/SomePage/SubPage.html\">SomePage/SubPage</a>"
    }

    do_request( "wiki.rb/SomePage/SubPage.html" )
    
    Web::local.assert_includes "page" => {
      "name" => "SomePage/SubPage",
      "html" => "make this exist"
    }

  end

end
