require 'web/wiki'

class TestTarpit < Test::Unit::TestCase
  include Web::Testing

  def setup
    @old_remote_addr = ENV["REMOTE_ADDR"]
    setup_wiki
  end

  def teardown
    teardown_wiki
    ENV["REMOTE_ADDR"] = @old_remote_addr
    File.delete("vandals.test.txt") if File.exists? "vandals.test.txt"
  end

  def do_edit( name, content )
    do_request "wiki.rb", "action" => "edit", "page.name" => name
    do_submit "edit_page", "page.content" => content, "action" => "Save"
  end

  def test_page_revisions_store_remote_addr
    page = Web::Wiki.wiki.load_page( "HomePage" )
    assert_equal( "auto-create", page.remote_addr )

    ENV["REMOTE_ADDR"] = "127.0.0.1"
    do_edit( "HomePage", "this is a test" )
    page = Web::Wiki.wiki.load_page( "HomePage" )
    assert_equal( "127.0.0.1", page.remote_addr )
  end

  def test_tarpit
     File.open( "vandals.test.txt", "w" ) { |f|
       f.puts( "123.51.23.192", "127.0.0.1", "52.213.123.65" )
     }
     ENV["REMOTE_ADDR"] = "127.0.0.1"
     assert_equal( File::expand_path(make_temp("tarpit")),
                   Web::Wiki.store_dir.gsub(/\/$/, '') )
  end

  def test_ip_range
    File.open( "vandals.test.txt", "w" ) { |f|
       f.puts( "123.51", "23.17.49", "88.34.23.9")
    }

    assert_equal( make_temp("test.pages/"),
                  Web::Wiki::store_dir )

    ["123.51.27.13", "23.17.49.2", "88.34.23.9"].each{ |ip|
      ENV["REMOTE_ADDR"] = ip
      assert_equal( make_temp("tarpit"),
                    Web::Wiki.store_dir.gsub(/\/$/, '') )
    }
  end
  
end
