require 'web/wiki'

class WikiTest < Test::Unit::TestCase
  include Web::Testing

  def test_default
    do_request "wiki.rb"
    Web::local.assert_includes "page" => { "name" => "HomePage", "html" => ""}
    assert_template_used( "home.html" )
  end

  # this is todo b/c it needs a little testing framework, i.e.
  # the other.html file must exist.  and I don't want to make it
  # exist permanently
  def todo_test_change_default
    Web::Wiki::set_pref( :home_page     => "OtherPage",
                         :home_template => "other.html" )
    do_request "wiki.rb"
    Web::local.assert_includes "page" => { "name" => "OtherPage", "html" => ""}
    assert_template_used( "other.html" )

    Web::Wiki::set_pref( :home_page     => "OtherPage",
                         :home_template => "home.html" )
    do_request "wiki.rb"
    Web::local.assert_includes "page" => { "name" => "OtherPage"}
    assert_template_used( "home.html" )

    Web::Wiki::set_pref( :home_page     => "HomePage",
                         :home_template => "home.html" )
  end

  def do_edit( name, content )
    do_request "wiki.rb", "action" => "edit", "page.name" => name
    do_submit "edit_page", "page.content" => content, "action" => "Save"
  end

  def do_edit_test( name, content )
    do_request "wiki.rb", "page.name" => name
    Web::local.assert_includes "page" => { "name" => name, "html" => ""}
    assert_template_used( "basic.html" )

    do_request "wiki.rb", "action" => "edit", "page.name" => name
    assert_template_used( "edit.html" )
    assert_edit_form(name)

    do_submit "edit_page", "page.content" => content, "action" => "Save"
    assert_template_used( "basic.html" )
    Web::local.assert_includes "page" => { "name" => name, "html" => content }
  end

  def test_circumvent_yaml_bugs
    do_edit_test( "SomePage", "Hello\"\'\r\nPatrick" )

    do_request "wiki.rb", "page.name" => "SomePage"
    
    Web::local.assert_includes "page" => {
      "name" => "SomePage",
      "html" =>"Hello\"\'\r\nPatrick",
      "content" => "Hello\"\'\r\nPatrick",
    }
  end

  def test_edit_somepage
      do_edit_test( "SomePage", "Hello Patrick" )
  end

  def test_do_edit_with_0_revision
    do_request "wiki.rb", "page.name" => "HomePage", "revision" => "0", "action" => "edit"
    assert_template_used( "edit.html" )
    assert_edit_form("HomePage")
  end

  def test_edit_different_page
      do_edit_test( "differentPage", "Hello Patrick" )
  end

  def test_page_has_a_time
    page = Web::Wiki.wiki.load_page( "HomePage" )
    mtime0 = page.mtime
    assert_equal( Time, mtime0.class )

    do_edit("HomePage", "first edits")

    page = Web::Wiki.wiki.load_page( "HomePage" )
    mtime1 = page.mtime
    assert_equal( Time, mtime1.class )
    assert( mtime0 < mtime1 )

    do_edit("HomePage", "second edits")
    
    page = Web::Wiki.wiki.load_page( "HomePage" )
    mtime2 = page.mtime
    assert_equal( Time, mtime2.class )
    assert( mtime0 < mtime1 && mtime1 < mtime2 )
  end
  
  def test_pretty_time
    mtime = Time.mktime(1977, 10, 20, 13, 30, 0, 0 )
    page = Web::Wiki.wiki.load_page( "HomePage" )
    page.mtime = mtime
    assert_equal( "1:30pm Oct 20, 1977", page.mtime_pretty )
  end


  def assert_edit_form( name="HomePage" )
    assert_form_includes( "edit_page",
			  "page" => {
			   "name" => name,
			   "content" => ""
			 } )
  end

  def test_use_different_edit_templates
    do_request( "wiki.rb",
                "page.name" => "HomePage",
                "action"    => "edit" )
    assert_template_used( "default-content_editor.html" )
    assert_edit_form

    do_request( "wiki.rb",
                "page.name" => "HomePage",
                "action"    => "edit",
                :env => {"HTTP_USER_AGENT" => "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)"})
    assert_template_used( "ie-content_editor.html" )
    assert_edit_form
  
    do_request( "wiki.rb",
                "page.name" => "HomePage",
                "action"    => "edit",
                :env => {"HTTP_USER_AGENT" => "Mozilla/5.001 (Windows; U; NT4.0; en-us) Gecko /25250101"} )
    assert_template_used( "ekit-content_editor.html" )
    assert_edit_form
  end
  
  def test_look_and_feel_template
    do_request( "wiki.rb", "page.name" => "HomePage" )
    assert_template_used( "template.html" )
    assert_template_used( "home.html" )
    
    do_edit("SomePage", "edits")
    assert_template_used( "template.html" )
    assert_template_used( "basic.html" )
    
    #do_request "wiki.rb", "action" => "edit", "page.name" => "Some Page"
    #do_submit( "edit_page",
    #           "page.illustration" => "http://www.somewhere.com/photo.gif",
    #           "action" => "Save" )

    
    #assert_template_used( "illustration.html" )
    #assert_template_not_used( "template.html" )

      do_request("wiki.rb",
                 "page.name" => "HomePage",
                 "action" => "images")
      assert_template_used( "images.html" )
      assert_template_not_used( "template.html" )
  end

  def test_admin_template
    do_request( "admin.rb", "page.name" => "HomePage" )
    assert_template_used( "admin.html" )
    assert_template_used( "home.html" )
    
    do_edit("SomePage", "edits")

    #do_request "admin.rb", "action" => "edit", "page.name" => "Some Page"
    #do_submit( "edit_page",
   #            "page.illustration" => "http://www.somewhere.com/photo.gif",
   #            "action" => "Save" )

    
    #assert_template_used( "illustration.html" )
    #assert_template_not_used( "admin.html" )

      do_request("admin.rb",
                 "page.name" => "HomePage",
                 "action" => "images")
      assert_template_used( "images.html" )
      assert_template_not_used( "admin.html" )
  end

  def test_preferences
    assert_equal( nil, Web::Wiki::pref("random_pref") )
    Web::Wiki::set_pref("random_pref" => "random value",
                        "another_pref" => "another value")
    assert_equal( "random value", Web::Wiki::pref("random_pref") )
    assert_equal( "random value", Web::Wiki::preferences["random_pref"] )
    assert_equal( "another value", Web::Wiki::pref("another_pref") )
    assert_equal( "another value", Web::Wiki::preferences["another_pref"] )
    begin
      assert_equal( make_temp("test.pages/"), Web::Wiki::pref("store_dir") )
      Web::Wiki::set_pref("store_dir" => "da_sto")
      assert_equal( "da_sto", Web::Wiki::pref("store_dir") )
      assert_equal( "da_sto", Web::Wiki::preferences["store_dir"] )
    ensure
      Web::Wiki::set_pref("store_dir"     => make_temp("/test.pages/"),
                          "random_pref"  => nil,
                          "another_pref" => nil)
    end
  end
  
  def test_allowed_actions
    all_actions = ["page_history",
                   "download",
                   "default",
                   "download_resource",
                   "more_news",
                   "edit",
                   "Save",
                   "images",
                   "asset_history",
                   "select_illustration",
                   "insert_download",
                   "Upload",
                   "delete_asset"]
    assert_equal( all_actions.sort, Web::Wiki::pref("allowed_actions").sort )
  end

  def test_restrict_actions
    all_actions = Web::Wiki::pref("allowed_actions")
    begin
      Web::Wiki::set_pref("allowed_actions" => ["download",
                                                "more_news"] )
      do_request "wiki.rb"
      Web::local.assert_includes "page" => { "name" => "HomePage", "html" => ""}
      assert_template_used( "home.html" )
      
      do_request "wiki.rb", "action" => "edit"
      Web::local.assert_includes "page" => { "name" => "HomePage", "html" => ""}
      assert_template_used( "home.html" )
      assert_template_not_used( "edit.html" )
      
      Web::Wiki::set_pref("allowed_actions" => ["download",
                                                "more_news",
                                                "edit", ] )
      
      do_request "wiki.rb", "action" => "edit"
      Web::local.assert_includes "page" => { "name" => "HomePage", "html" => ""}
      assert_template_used( "edit.html" )
      assert_template_not_used( "home.html" )
    ensure
      Web::Wiki::set_pref("allowed_actions" => all_actions)
    end
  end

  def test_download_resource
    do_request "wiki.rb/resources/test_upload.gif"

    expected_asset = nil
    File.open( "lib/web/wiki/resources/test_upload.gif", "r" ) { |f|
      expected_asset = f.read
    }

    assert_equal( expected_asset, Web.connection.get_content )
    assert_equal( ["image/gif"], Web.connection.get_header("Content-Type") )

    do_request "wiki.rb/resources/htmlarea/images/insert_table.gif"

    expected_asset = nil
    File.open( "lib/web/wiki/resources/htmlarea/images/insert_table.gif", "r" ) { |f|
      expected_asset = f.read
    }

    assert_equal( expected_asset, Web.connection.get_content )
    assert_equal( ["image/gif"], Web.connection.get_header("Content-Type") )

    # handle bad links appropriately
    do_request "wiki.rb/resources/non_existent_file.gif"
    assert_equal( "404 File Not Found", Web.connection.get_content )
    assert_equal( ["404 Not Found"], Web.connection.get_header("Status") )


    # don't allow this
    do_request "wiki.rb/resources/../wiki.rb"
    assert_equal( "404 File Not Found", Web.connection.get_content )
    assert_equal( ["404 Not Found"], Web.connection.get_header("Status") )
  end

  def test_default_preferences
    assert_equal( "wiki.rb", Web::Wiki::pref("baseurl") )
    do_request( "wiki.rb" )
    Web::local.assert_includes( "baseurl" => "wiki.rb" )

    assert_equal( "wiki.rb/resources", Web::Wiki::pref("resourceurl") )
    do_request( "wiki.rb" )
    Web::local.assert_includes( "resourceurl" => "wiki.rb/resources" )
  end

  def setup
    Web::setup
  end
  
  def teardown
    Web::teardown
    teardown_wiki
  end

end

